@@ -5,11 +5,12 @@ from math import exp
 
 import array
 
-from ..traitlets import Float, Int, Instance, AfgenTrait, Bool
+from ..traitlets import Float, Int, Instance, Bool
 from ..decorators import prepare_rates, prepare_states
-from ..base_classes import ParamTemplate, StatesTemplate, RatesTemplate, \
+from ..base import ParamTemplate, StatesTemplate, RatesTemplate, \
                          SimulationObject
-from ..util import limit, merge_dict
+from ..util import limit, merge_dict, AfgenTrait
+
 
 def SWEAF(ET0, DEPNR):
     """Calculates the Soil Water Easily Available Fraction (SWEAF).
@@ -27,10 +28,10 @@ def SWEAF(ET0, DEPNR):
     """
     A = 0.76
     B = 1.5
-#   curve for CGNR 5, and other curves at fixed distance below it
+    # curve for CGNR 5, and other curves at fixed distance below it
     sweaf = 1./(A+B*ET0) - (5.-DEPNR)*0.10
 
-#   Correction for lower curves (CGNR less than 3)
+    # Correction for lower curves (CGNR less than 3)
     if (DEPNR < 3.):
         sweaf += (ET0-0.6)/(DEPNR*(DEPNR+3.))
 
@@ -38,7 +39,8 @@ def SWEAF(ET0, DEPNR):
 
 
 class Evapotranspiration(SimulationObject):
-    """Calculation of evaporation (water and soil) and transpiration rates.
+    """Calculation of potential evaporation (water and soil) rates and actual
+    crop transpiration rate.
         
     *Simulation parameters*:
     
@@ -86,8 +88,11 @@ class Evapotranspiration(SimulationObject):
     EVSMX    Maximum evaporation rate from a wet soil surface.  Y    |cm day-1|
     TRAMX    Maximum transpiration rate from the plant canopy   Y    |cm day-1|
     TRA      Actual transpiration rate from the plant canopy    Y    |cm day-1|
-    IDOS     Indicates oxygen stress on this day (True|False)   N    -
-    IDWS     Indicates water stress on this day (True|False)    N    -
+    IDOS     Indicates oxygen stress on this day (True|False)   N     -
+    IDWS     Indicates water stress on this day (True|False)    N     -
+    RFWS     Reduction factor for water stress                  N     -
+    RFOS     Reduction factor for oxygen stress                 N     -
+    RFTRA    Reduction factor for transpiration (wat & ox)      Y     -
     =======  ================================================= ==== ============
     
     *Signals send or handled*
@@ -105,34 +110,36 @@ class Evapotranspiration(SimulationObject):
     =======  =================================== =================  ============
     """
 
-    # helper variable for Counting days since oxygen stress (DSOS)
-    # and total days with water and oxygen stress (IDWST, IDOST)
-    _DSOS = Int(-99)
-    _IDWST = Int(-99)
-    _IDOST = Int(-99)
+    # helper variable for Counting total days with water and oxygen
+    # stress (IDWST, IDOST)
+    _IDWST = Int(0)
+    _IDOST = Int(0)
 
     class Parameters(ParamTemplate):
-        CFET   = Float(-99.)
-        DEPNR  = Float(-99.)
+        CFET = Float(-99.)
+        DEPNR = Float(-99.)
         KDIFTB = AfgenTrait()
         IAIRDU = Float(-99.)
-        IOX    = Float(-99.)
+        IOX = Float(-99.)
         CRAIRC = Float(-99.)
-        SM0    = Float(-99.)
-        SMW    = Float(-99.)
-        SMFCF  = Float(-99.)
+        SM0 = Float(-99.)
+        SMW = Float(-99.)
+        SMFCF = Float(-99.)
 
     class RateVariables(RatesTemplate):
         EVWMX = Float(-99.)
         EVSMX = Float(-99.)
         TRAMX = Float(-99.)
-        TRA   = Float(-99.)
-        IDOS  = Bool(False)
-        IDWS  = Bool(False)
+        TRA = Float(-99.)
+        IDOS = Bool(False)
+        IDWS = Bool(False)
+        RFWS = Float(-99.)
+        RFOS = Float(-99.)
+        RFTRA = Float(-99.)
 
     class StateVariables(StatesTemplate):
-        IDOST  = Int(-99)
-        IDWST  = Int(-99)
+        IDOST = Int(-99)
+        IDWST = Int(-99)
 
     def initialize(self, day, kiosk, parvalues):
         """
@@ -144,76 +151,54 @@ class Evapotranspiration(SimulationObject):
 
         self.kiosk = kiosk
         self.params = self.Parameters(parvalues)
-        self.rates = self.RateVariables(kiosk, publish=["EVWMX", "EVSMX",
-                                                        "TRAMX", "TRA"
-                                                        ])
+        self.rates = self.RateVariables(kiosk, publish=["EVWMX", "EVSMX", "TRAMX", "TRA", "RFTRA"])
         self.states = self.StateVariables(kiosk, IDOST=-999, IDWST=-999)
-        
-        # Helper variables
-        self._DSOS = 0
-        self._IDWST = 0
-        self._IDOST = 0
 
     @prepare_rates
     def __call__(self, day, drv):
         p = self.params
         r = self.rates
-        s = self.states
+        k = self.kiosk
         
-        DVS = self.kiosk["DVS"]
-        LAI = self.kiosk["LAI"]
-        SM  = self.kiosk["SM"]
-
-        KGLOB = 0.75*p.KDIFTB(DVS)
+        KGLOB = 0.75 * p.KDIFTB(k.DVS)
   
         # crop specific correction on potential transpiration rate
-        ET0 = p.CFET * drv.ET0
+        ET0_CROP = max(0., p.CFET * drv.ET0)
   
         # maximum evaporation and transpiration rates
-        EKL = exp(-KGLOB * LAI)
+        EKL = exp(-KGLOB * k.LAI)
         r.EVWMX = drv.E0 * EKL
         r.EVSMX = max(0., drv.ES0 * EKL)
-        r.TRAMX = max(0.000001, ET0 * (1.-EKL))
+        r.TRAMX = ET0_CROP * (1.-EKL)
                 
         # Critical soil moisture
-        SWDEP = SWEAF(ET0, p.DEPNR)
+        SWDEP = SWEAF(ET0_CROP, p.DEPNR)
         SMCR = (1.-SWDEP)*(p.SMFCF-p.SMW) + p.SMW
 
         # Reduction factor for transpiration in case of water shortage (RFWS)
-        RFWS = limit(0., 1., (SM-p.SMW)/(SMCR-p.SMW))
+        r.RFWS = limit(0., 1., (k.SM - p.SMW)/(SMCR - p.SMW))
 
         # reduction in transpiration in case of oxygen shortage (RFOS)
         # for non-rice crops, and possibly deficient land drainage
-        if (p.IAIRDU == 0 and p.IOX == 1):
-            # critical soil moisture content for aeration
-            SMAIR = p.SM0 - p.CRAIRC
-
-            # count days since start oxygen shortage (up to 4 days)
-            if SM >= SMAIR:
-                self._DSOS = min((self._DSOS+1),4)
-            else:
-                self._DSOS = 0
-
+        r.RFOS = 1.
+        if p.IAIRDU == 0 and p.IOX == 1:
+            RFOSMX = limit(0., 1., (p.SM0 - k.SM)/p.CRAIRC)
             # maximum reduction reached after 4 days
-            RFOSMX = limit(0., 1., (p.SM0-SM)/(p.SM0-SMAIR))
-            RFOS   = RFOSMX + (1. - self._DSOS/4.)*(1.-RFOSMX)
+            r.RFOS = RFOSMX + (1. - min(k.DSOS, 4)/4.)*(1.-RFOSMX)
 
-        # For rice, or non-rice crops grown on well drained land
-        elif (p.IAIRDU == 1 or p.IOX == 0):
-            RFOS = 1.
-        # Transpiration rate multiplied with reduction factors for oxygen and
-        # water
-        r.TRA = r.TRAMX * RFOS * RFWS
+        # Transpiration rate multiplied with reduction factors for oxygen and water
+        r.RFTRA = r.RFOS * r.RFWS
+        r.TRA = r.TRAMX * r.RFTRA
 
         # Counting stress days
-        if RFWS < 1.:
+        if r.RFWS < 1.:
             r.IDWS = True
             self._IDWST += 1
-        if RFOS < 1.:
+        if r.RFOS < 1.:
             r.IDOS = True
             self._IDOST += 1
 
-        return (r.TRA, r.TRAMX)
+        return r.TRA, r.TRAMX
         
     @prepare_states
     def finalize(self, day):
@@ -223,6 +208,7 @@ class Evapotranspiration(SimulationObject):
         
         SimulationObject.finalize(self, day)
 
+
 class EvapotranspirationLayered(SimulationObject):
     """Calculation of evaporation (water and soil) and transpiration rates
     for a layered soil.
@@ -431,6 +417,186 @@ class EvapotranspirationLayered(SimulationObject):
         SimulationObject.finalize(self, day)
 
 
+class EvapotranspirationCO2(SimulationObject):
+    """Calculation of evaporation (water and soil) and transpiration rates
+    taking into account the CO2 effect on crop transpiration.
+
+    *Simulation parameters* (To be provided in cropdata dictionary):
+
+    ======== ============================================= =======  ============
+     Name     Description                                   Type     Unit
+    ======== ============================================= =======  ============
+    CFET     Correction factor for potential transpiration   S       -
+             rate.
+    DEPNR    Dependency number for crop sensitivity to       S       -
+             soil moisture stress.
+    KDIFTB   Extinction coefficient for diffuse visible      T       -
+             as function of DVS.
+    IOX      Switch oxygen stress on (1) or off (0)          S       -
+    IAIRDU   Switch airducts on (1) or off (0)               S       -
+    CRAIRC   Critical air content for root aeration          S       -
+    SM0      Soil porosity                                   S       -
+    SMW      Volumetric soil moisture content at wilting     S       -
+             point
+    SMCFC    Volumetric soil moisture content at field       S       -
+             capacity
+    SM0      Soil porosity                                   S       -
+    CO2      Atmospheric CO2 concentration                   S       ppm
+    CO2TRATB Reduction factor for TRAMX as function of
+             atmospheric CO2 concentration                   T       -
+    ======== ============================================= =======  ============
+
+
+    *State variables*
+
+    Note that these state variables are only assigned after finalize() has been
+    run.
+
+    =======  ================================================= ==== ============
+     Name     Description                                      Pbl      Unit
+    =======  ================================================= ==== ============
+    IDWST     Nr of days with water stress.                      N    -
+    IDOST     Nr of days with oxygen stress.                     N    -
+    =======  ================================================= ==== ============
+
+
+    *Rate variables*
+
+    =======  ================================================= ==== ============
+     Name     Description                                      Pbl      Unit
+    =======  ================================================= ==== ============
+    EVWMX    Maximum evaporation rate from an open water        Y    |cm day-1|
+             surface.
+    EVSMX    Maximum evaporation rate from a wet soil surface.  Y    |cm day-1|
+    TRAMX    Maximum transpiration rate from the plant canopy   Y    |cm day-1|
+    TRA      Actual transpiration rate from the plant canopy    Y    |cm day-1|
+    IDOS     Indicates water stress on this day (True|False)    N    -
+    IDWS     Indicates oxygen stress on this day (True|False)   N    -
+    RFWS     Reducation factor for water stress                 Y     -
+    RFOS     Reducation factor for oxygen stress                Y     -
+    RFTRA    Reduction factor for transpiration (wat & ox)      Y     -
+    =======  ================================================= ==== ============
+
+    *Signals send or handled*
+
+    None
+
+    *External dependencies:*
+
+    =======  =================================== =================  ============
+     Name     Description                         Provided by         Unit
+    =======  =================================== =================  ============
+    DVS      Crop development stage              DVS_Phenology       -
+    LAI      Leaf area index                     Leaf_dynamics       -
+    SM       Volumetric soil moisture content    Waterbalance        -
+    =======  =================================== =================  ============
+    """
+
+    # helper variable for counting total days with water and oxygen
+    # stress (IDWST, IDOST)
+    _IDWST = Int(0)
+    _IDOST = Int(0)
+
+    class Parameters(ParamTemplate):
+        CFET    = Float(-99.)
+        DEPNR   = Float(-99.)
+        KDIFTB  = AfgenTrait()
+        IAIRDU  = Float(-99.)
+        IOX     = Float(-99.)
+        CRAIRC  = Float(-99.)
+        SM0     = Float(-99.)
+        SMW     = Float(-99.)
+        SMFCF   = Float(-99.)
+        CO2     = Float(-99.)
+        CO2TRATB = AfgenTrait()
+
+    class RateVariables(RatesTemplate):
+        EVWMX = Float(-99.)
+        EVSMX = Float(-99.)
+        TRAMX = Float(-99.)
+        TRA   = Float(-99.)
+        TRALY = Instance(array.array)
+        IDOS  = Bool(False)
+        IDWS  = Bool(False)
+        RFWS = Float(-99.)
+        RFOS = Float(-99.)
+        RFTRA = Float(-99.)
+
+    class StateVariables(StatesTemplate):
+        IDOST  = Int(-99)
+        IDWST  = Int(-99)
+
+    def initialize(self, day, kiosk, parvalues):
+        """
+        :param day: start date of the simulation
+        :param kiosk: variable kiosk of this PCSE instance
+        :param cropdata: dictionary with WOFOST cropdata key/value pairs
+        :param soildata: dictionary with WOFOST soildata key/value pairs
+        """
+
+        self.kiosk = kiosk
+        self.params = self.Parameters(parvalues)
+        self.rates = self.RateVariables(kiosk, publish=["EVWMX","EVSMX", "TRAMX","TRA","TRALY", "RFTRA"])
+        self.states = self.StateVariables(kiosk, IDOST=-999, IDWST=-999)
+
+    @prepare_rates
+    def __call__(self, day, drv):
+        p = self.params
+        r = self.rates
+        k = self.kiosk
+
+        # reduction factor for CO2 on TRAMX
+        RF_TRAMX_CO2 = p.CO2TRATB(p.CO2)
+
+        # crop specific correction on potential transpiration rate
+        ET0_CROP = max(0., p.CFET * drv.ET0)
+
+        # maximum evaporation and transpiration rates
+        KGLOB = 0.75*p.KDIFTB(k.DVS)
+        EKL = exp(-KGLOB * k.LAI)
+        r.EVWMX = drv.E0 * EKL
+        r.EVSMX = max(0., drv.ES0 * EKL)
+        r.TRAMX = ET0_CROP * (1.-EKL) * RF_TRAMX_CO2
+
+        # Critical soil moisture
+        SWDEP = SWEAF(ET0_CROP, p.DEPNR)
+
+        SMCR = (1.-SWDEP)*(p.SMFCF-p.SMW) + p.SMW
+
+        # Reduction factor for transpiration in case of water shortage (RFWS)
+        r.RFWS = limit(0., 1., (k.SM-p.SMW)/(SMCR-p.SMW))
+
+        # reduction in transpiration in case of oxygen shortage (RFOS)
+        # for non-rice crops, and possibly deficient land drainage
+        r.RFOS = 1.
+        if p.IAIRDU == 0 and p.IOX == 1:
+            RFOSMX = limit(0., 1., (p.SM0 - k.SM)/p.CRAIRC)
+            # maximum reduction reached after 4 days
+            r.RFOS = RFOSMX + (1. - min(k.DSOS, 4)/4.)*(1.-RFOSMX)
+
+        # Transpiration rate multiplied with reduction factors for oxygen and water
+        r.RFTRA = r.RFOS * r.RFWS
+        r.TRA = r.TRAMX * r.RFTRA
+
+        # Counting stress days
+        if r.RFWS < 1.:
+            r.IDWS = True
+            self._IDWST += 1
+        if r.RFOS < 1.:
+            r.IDOS = True
+            self._IDOST += 1
+
+        return r.TRA, r.TRAMX
+
+    @prepare_states
+    def finalize(self, day):
+
+        self.states.IDWST = self._IDWST
+        self.states.IDOST = self._IDOST
+
+        SimulationObject.finalize(self, day)
+
+
 class Simple_Evapotranspiration(SimulationObject):
     """Calculation of evaporation (water and soil) and transpiration rates.
     
@@ -486,6 +652,8 @@ class Simple_Evapotranspiration(SimulationObject):
         SM0    = Float(-99.)
         SMW    = Float(-99.)
         SMFCF  = Float(-99.)
+        CFET = Float(-99.)
+        DEPNR = Float(-99.)
 
     class RateVariables(RatesTemplate):
         EVWMX = Float(-99.)
@@ -518,9 +686,7 @@ class Simple_Evapotranspiration(SimulationObject):
         KGLOB = 0.75*KDIF
   
         # crop specific correction on potential transpiration rate
-        # CFET set to 1.
-        CFET = 1.
-        ET0 = CFET * drv.ET0
+        ET0 = p.CFET * drv.ET0
   
         # maximum evaporation and transpiration rates
         EKL = exp(-KGLOB * LAI)
@@ -529,9 +695,7 @@ class Simple_Evapotranspiration(SimulationObject):
         r.TRAMX = max(0.000001, ET0 * (1.-EKL))
         
         # Critical soil moisture
-        # set DEPNR to 4.5 (typical for cereals)
-        DEPNR = 4.5
-        SWDEP = SWEAF(ET0, DEPNR)
+        SWDEP = SWEAF(ET0, p.DEPNR)
         
         SMCR = (1.-SWDEP)*(p.SMFCF-p.SMW) + p.SMW
 
