@@ -9,11 +9,11 @@ Classes defined here:
 """
 import datetime
 
-from ..traitlets import Float, Int, Instance, Enum, Bool, AfgenTrait
+from ..traitlets import Float, Int, Instance, Enum, Bool
 from ..decorators import prepare_rates, prepare_states
 
-from ..util import limit, daylength
-from ..base_classes import ParamTemplate, StatesTemplate, RatesTemplate, \
+from ..util import limit, daylength, AfgenTrait
+from ..base import ParamTemplate, StatesTemplate, RatesTemplate, \
      SimulationObject, VariableKiosk
 from .. import signals
 from .. import exceptions as exc
@@ -154,7 +154,7 @@ class Vernalisation(SimulationObject):
             rates.VERNFAC = 1.0
     #---------------------------------------------------------------------------
     @prepare_states
-    def integrate(self, day):
+    def integrate(self, day, delt=1.0):
         states = self.states
         rates = self.rates
         params = self.params
@@ -293,8 +293,8 @@ class DVS_Phenology(SimulationObject):
 
     #-------------------------------------------------------------------------------
     class StateVariables(StatesTemplate):
-        DVS   = Float(-99.)  # Development stage
-        TSUM  = Float(-99.)  # Temperature sum state
+        DVS = Float(-99.)  # Development stage
+        TSUM = Float(-99.)  # Temperature sum state
         TSUME = Float(-99.)  # Temperature sum for emergence state
         # States which register phenological events
         DOS = Instance(datetime.date) # Day of sowing
@@ -302,7 +302,7 @@ class DVS_Phenology(SimulationObject):
         DOA = Instance(datetime.date) # Day of anthesis
         DOM = Instance(datetime.date) # Day of maturity
         DOH = Instance(datetime.date) # Day of harvest
-        STAGE  = Enum([None, "emerging", "vegetative", "reproductive", "mature"])
+        STAGE = Enum(["emerging", "vegetative", "reproductive", "mature"])
 
     #---------------------------------------------------------------------------
     def initialize(self, day, kiosk, parvalues):
@@ -320,8 +320,7 @@ class DVS_Phenology(SimulationObject):
         self._connect_signal(self._on_CROP_FINISH, signal=signals.crop_finish)
 
         # Define initial states
-        DVS = self.params.DVSI
-        DOS, DOE, STAGE = self._get_initial_stage(day)
+        DVS, DOS, DOE, STAGE = self._get_initial_stage(day)
         self.states = self.StateVariables(kiosk, publish="DVS",
                                           TSUM=0., TSUME=0., DVS=DVS,
                                           DOS=DOS, DOE=DOE, DOA=None, DOM=None,
@@ -342,6 +341,7 @@ class DVS_Phenology(SimulationObject):
             STAGE = "vegetative"
             DOE = day
             DOS = None
+            DVS = p.DVSI
 
             # send signal to indicate crop emergence
             self._send_signal(signals.crop_emerged)
@@ -350,12 +350,13 @@ class DVS_Phenology(SimulationObject):
             STAGE = "emerging"
             DOS = day
             DOE = None
+            DVS = -0.1
 
         else:
             msg = "Unknown start type: %s" % p.CROP_START_TYPE
             raise exc.PCSEError(msg)
             
-        return (DOS, DOE, STAGE)
+        return DVS, DOS, DOE, STAGE
 
     #---------------------------------------------------------------------------
     @prepare_rates
@@ -375,44 +376,39 @@ class DVS_Phenology(SimulationObject):
         # Vernalisation
         VERNFAC = 1.
         if p.IDSL >= 2:
-            self.vernalisation.calc_rates(day, drv)
-            VERNFAC = self.kiosk["VERNFAC"]
+            if s.STAGE == 'vegetative':
+                self.vernalisation.calc_rates(day, drv)
+                VERNFAC = self.kiosk["VERNFAC"]
 
+        # Development rates
         if s.STAGE == "emerging":
             r.DTSUME = limit(0., (p.TEFFMX - p.TBASEM), (drv.TEMP - p.TBASEM))
-            
-        elif s.STAGE == 'vegetative':
-            # Temperature sum increase
-            DTSUM = p.DTSMTB(drv.TEMP)
+            r.DTSUM = 0.
+            r.DVR = 0.1 * r.DTSUME/p.TSUMEM
 
-            # Development rate
-            r.DTSUM = DTSUM * VERNFAC * DVRED
+        elif s.STAGE == 'vegetative':
+            r.DTSUME = 0.
+            r.DTSUM = p.DTSMTB(drv.TEMP) * VERNFAC * DVRED
             r.DVR = r.DTSUM/p.TSUM1
 
         elif s.STAGE == 'reproductive':
-            # Temperature sum increase
+            r.DTSUME = 0.
             r.DTSUM = p.DTSMTB(drv.TEMP)
-
-            # Development rate
             r.DVR = r.DTSUM/p.TSUM2
-            
         elif s.STAGE == 'mature':
-            # Temperature sum increase
-            r.DTSUM = p.DTSMTB(drv.TEMP)
-
-            # Development rate
-            r.DVR = r.DTSUM/p.TSUM2
-            
-        else: # Problem: no stage defined
-            msg = "No STAGE defined in phenology submodule"
-            raise exc.PCSEError(msg)
+            r.DTSUME = 0.
+            r.DTSUM = 0.
+            r.DVR = 0.
+        else:  # Problem: no stage defined
+            msg = "Unrecognized STAGE defined in phenology submodule: %s"
+            raise exc.PCSEError(msg, self.states.STAGE)
         
         msg = "Finished rate calculation for %s"
         self.logger.debug(msg % day)
         
     #---------------------------------------------------------------------------
     @prepare_states
-    def integrate(self, day):
+    def integrate(self, day, delt=1.0):
         """Updates the state variable and checks for phenologic stages
         """
 
@@ -422,30 +418,31 @@ class DVS_Phenology(SimulationObject):
 
         # Integrate vernalisation module
         if p.IDSL >= 2:
-            self.vernalisation.integrate(day)
+            if s.STAGE == 'vegetative':
+                self.vernalisation.integrate(day, delt)
+            else:
+                self.vernalisation.touch()
 
+        # Integrate phenologic states
+        s.TSUME += r.DTSUME
+        s.DVS += r.DVR
+        s.TSUM += r.DTSUM
+
+        # Check if a new stage is reached
         if s.STAGE == "emerging":
-            s.TSUME += r.DTSUME
-            if s.TSUME >= p.TSUMEM:
+            if s.DVS >= 0.0:
                 self._next_stage(day)
-                
+                s.DVS = 0.
         elif s.STAGE == 'vegetative':
-            s.DVS += r.DVR
-            s.TSUM += r.DTSUM
             if s.DVS >= 1.0:
                 self._next_stage(day)
                 s.DVS = 1.0
-                
         elif s.STAGE == 'reproductive':
-            s.DVS += r.DVR
-            s.TSUM += r.DTSUM
             if s.DVS >= p.DVSEND:
                 self._next_stage(day)
-                
+                s.DVS = p.DVSEND
         elif s.STAGE == 'mature':
-            s.DVS += r.DVR
-            s.TSUM += r.DTSUM
-            
+            pass
         else: # Problem no stage defined
             msg = "No STAGE defined in phenology submodule"
             raise exc.PCSEError(msg)
@@ -475,7 +472,8 @@ class DVS_Phenology(SimulationObject):
             s.DOM = day
             if p.CROP_END_TYPE in ["maturity","earliest"]:
                 self._send_signal(signal=signals.crop_finish,
-                                  day=day, finish_type="maturity")
+                                  day=day, finish_type="maturity",
+                                  crop_delete=True)
         elif s.STAGE == "mature":
             msg = "Cannot move to next phenology stage: maturity already reached!"
             raise exc.PCSEError(msg)
@@ -493,27 +491,5 @@ class DVS_Phenology(SimulationObject):
         strictly related to phenology (but to management) this is the most
         logical place to put it.
         """
-        if finish_type == 'harvest':
+        if finish_type in ['harvest', 'earliest']:
             self._for_finalize["DOH"] = day
-
-
-class DVS_Phenology_Wrapper(SimulationObject):
-    """This is wrapper class that wraps the DVS_phenology simulation object
-    in such a way that it can run directly in the Engine. This means that
-    it can be used direct in a configuration file::
-
-        CROP = DVS_Phenology_Wrapper
-
-    This is useful for running only the phenology instead of the entire
-    crop simulation.
-    """
-    phenology = Instance(SimulationObject)
-
-    def initialize(self, day, kiosk, cropdata, soildata, sitedata, start_type, stop_type):
-        self.phenology = DVS_Phenology(day, kiosk, cropdata, start_type,  stop_type)
-
-    def calc_rates(self, day, drv):
-        self.phenology.calc_rates(day, drv)
-
-    def integrate(self, day):
-        self.phenology.integrate(day)
