@@ -1,11 +1,15 @@
 # -*- coding: utf-8 -*-
 # Copyright (c) 2004-2014 Alterra, Wageningen-UR
 # Allard de Wit (allard.dewit@wur.nl), April 2014
+"""Tools for reading from the internal PCSE demo database.
+
+Mainly used for running the WOFOST unit tests on the internal demo database.
+"""
 from .database_definition import migrate_db
 from .db_input import fetch_cropdata
 from .db_input import fetch_soildata_layered
 from .db_input import fetch_soildata
-from .db_input import fetch_timerdata
 from .db_input import fetch_sitedata
-from .db_input import GridWeatherDataProvider
-from .db_input import EnsembleGridWeatherDataProvider
\ No newline at end of file
+from .db_input import EnsembleGridWeatherDataProvider
+from .db_input import AgroManagementDataProvider
+from ..cgms8 import GridWeatherDataProvider
\ No newline at end of file
