@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # Copyright (c) 2004-2014 Alterra, Wageningen-UR
 # Allard de Wit (allard.dewit@wur.nl), April 2014
-"""Routines for retrieving data from the PyWofost database.
+"""Routines for retrieving data from the PCSE demo database.
  
 Implements the following functions:
     - fetch_cropdata()
@@ -21,18 +21,19 @@ Implements the following exceptions:
     - TimerdataError
 """
 import sys, os
-import datetime
+import datetime as dt
 import logging
 import copy
 import array
 from math import log10
 
 from sqlalchemy import create_engine, MetaData, select, Table, and_, join
-from sqlalchemy.sql.expression import func
+import yaml
 
-from ...util import wind10to2, Afgen
+from ...util import wind10to2, Afgen, check_date
 from ... import exceptions as exc
-from ...base_classes import WeatherDataContainer, WeatherDataProvider
+from ...base import WeatherDataContainer, WeatherDataProvider
+
 
 #-------------------------------------------------------------------------------
 class MeteodataError(exc.PCSEError):
@@ -64,13 +65,6 @@ class SoildataError(exc.PCSEError):
     def __str__(self):
          return repr(self.value)
 
-#-------------------------------------------------------------------------------
-class TimerdataError(exc.PCSEError):
-    def __init__(self, value):
-        self.value = value
-    def __str__(self):
-         return repr(self.value)
-
 #-------------------------------------------------------------------------------
 def fetch_cropdata(metadata, grid, year, crop):
     """Retrieve crop parameter values for given grid, year, crop from DB.
@@ -85,8 +79,8 @@ def fetch_cropdata(metadata, grid, year, crop):
     be extended when additional parameters are to be retrieved.
     """
     
-    # Define a logger for the PyWofost db_util routines
-    logger = logging.getLogger('PyWofost.db_util')
+    # Define a logger for the PCSE db_util routines
+    logger = logging.getLogger('PCSE.db_util')
 
     # Create initial dictionary 
     cropdata={}    
@@ -271,8 +265,8 @@ def fetch_soilparams(metadata, grid, soilgroup):
     raising a SoildataError
     """
     
-    # Define a logger for the PyWofost db_util routines
-    logger = logging.getLogger('PyWofost41.db_input') #Logger()
+    # Define a logger for the PCSE db_util routines
+    logger = logging.getLogger('PCSE41.db_input') #Logger()
 
     # Define soil physical variable parameter codes
     # defined as (code_parname, db_parname)
@@ -423,8 +417,8 @@ def fetch_soildata_layered(metadata, grid):
     Returns a dictionary with WOFOST soil parameter name/value pairs.
     """
     
-    # Define a logger for the PyWofost db_util routines
-    logger = logging.getLogger('PyWofost.db_util')
+    # Define a logger for the PCSE db_util routines
+    logger = logging.getLogger('PCSE.db_util')
     
     soildata = dict()
     table_soiltype = Table('soil_type', metadata, autoload=True)
@@ -478,8 +472,8 @@ def fetch_soildata(metadata, grid):
     Returns a dictionary with WOFOST soil parameter name/value pairs.
     """
     
-    # Define a logger for the PyWofost db_util routines
-    logger = logging.getLogger('PyWofost.db_util')
+    # Define a logger for the PCSE db_util routines
+    logger = logging.getLogger('PCSE.db_util')
     
     soildata = {}
     # Select soil from the table SOIL_TYPE
@@ -537,89 +531,120 @@ def fetch_soildata(metadata, grid):
     logger.info("Succesfully retrieved soil parameter values from database")
     return soildata
 
-#-------------------------------------------------------------------------------
-def fetch_timerdata(metadata, grid, year, crop):
-    """Retrieve timerdata from DB for given grid, year, crop.
+
+class AgroManagementDataProvider(list):
+    """Class for providing agromanagement data from the CROP_CALENDAR table in a PCSE database.
+
+    :param engine: SqlAlchemy engine object providing DB access
+    :param grid_no: Integer grid ID, maps to the grid_no column in the table
+    :param crop_no: Integer id of crop, maps to the crop_no column in the table
+    :param campaign_year: Integer campaign year, maps to the YEAR column in the table.
+           The campaign year refers to the year of the crop start. Thus for crops
+           crossing calendar years, the start_date can be in the previous year as the
+           harvest.
     
-    Routine retrieves the data related to the timer of the system (sowing data,
-    emergence date, start date of waterbalance) from the CROP_CALENDAR.
-    The routine verifies that the start data of the water balance (column
-    'start_day') is before or equal to the start date of the crop (column
-    'crop_start_day')
-    
-    Returns a dictionary with WOFOST timer parameter name/value pairs.
+    Note that this AgroManagementDataProvider is only used for the internal PCSE database
+    and not to be used for CGMS databases.
     """
+    agro_management_template = """
+          - {campaign_start_date}:
+                CropCalendar:
+                    crop_name: '{crop_name}'
+                    variety_name: '{variety_name}'
+                    crop_start_date: {crop_start_date}
+                    crop_start_type: {crop_start_type}
+                    crop_end_date: {crop_end_date}
+                    crop_end_type: {crop_end_type}
+                    max_duration: {duration}
+                TimedEvents: null
+                StateEvents: null
+        """
 
-    # Define a logger for the PyWofost db_util routines
-    logger = logging.getLogger('PyWofost.db_util')
+    def __init__(self, engine, grid_no, crop_no, campaign_year):
+        list.__init__(self)
+        self.grid_no = int(grid_no)
+        self.crop_no = int(crop_no)
+        self.campaign_year = int(campaign_year)
+        self.crop_name = "undefined"
 
-    # Make initial output dictionary
-    timerdata = {"GRID_NO":grid, "CROP_NO":crop, "CAMPAIGNYEAR":year}
+        metadata = MetaData(engine)
+        table_cc = Table("crop_calendar", metadata, autoload=True)
 
-    # Create and execute SQL query
-    try:
-        table_crop_calendar = Table('crop_calendar', metadata, autoload=True)
-        r = select([table_crop_calendar],
-                   and_(table_crop_calendar.c.grid_no==grid,
-                        table_crop_calendar.c.year==year,
-                        table_crop_calendar.c.crop_no==crop)).execute()
+        r = select([table_cc], and_(table_cc.c.grid_no == self.grid_no,
+                                    table_cc.c.crop_no == self.crop_no,
+                                    table_cc.c.year == self.campaign_year)).execute()
         row = r.fetchone()
         r.close()
-
         if row is None:
-            msg = "Entry in crop_calendar not found!"
-            raise RuntimeError
+            msg = "Failed deriving crop calendar for grid_no %s, crop_no %s " % (grid_no, crop_no)
+            raise exc.PCSEError(msg)
 
-        # Start day of the whole system. Verify that
-        # start_date <= crop_start_date
-        if row.crop_start_date < row.start_date:
-            msg = ("crop_start_date (%s) before system start_date (%s)" %
-                   (row.crop_start_date, row.start_date))
-            raise RuntimeError(msg)
-        timerdata["START_DATE"] = row.start_date
+        # Determine the start date.
+        self.crop_start_date = check_date(row.crop_start_date)
+        self.campaign_start_date = row.start_date
 
-        # Set start type
-        timerdata["CROP_START_TYPE"] = row.crop_start_type.lower()
+        # Determine the start date/type. Only sowing|emergence is accepted by PCSE/WOFOST
+        self.crop_start_type = str(row.crop_start_type).strip()
+        if self.crop_start_type not in ["sowing","emergence"]:
+            msg = "Unrecognized crop start type: %s" % self.crop_start_type
+            raise exc.PCSEError(msg)
 
-        # Set crop start day, it wil be used as emergence or sowing, 
-        # depending on CROP_START_TYPE
-        timerdata["CROP_START_DATE"] = row.crop_start_date
-    
-        # Set end of simulation type: fixed harvest or maturity
-        crop_end_type = row.crop_end_type.lower()
-        timerdata["CROP_END_TYPE"] = crop_end_type
-        timerdata["MAX_DURATION"] = row.max_duration
-        if (crop_end_type == "maturity") or (crop_end_type == "earliest"):
-            timerdata["CROP_END_DATE"] = row.crop_start_date + \
-                                         datetime.timedelta(days=row.max_duration)
-        elif (crop_end_type == "harvest"):
-            timerdata["CROP_END_DATE"] = row.crop_end_date
+        # Determine maximum duration of the crop
+        self.max_duration = int(row.max_duration)
+
+        # Determine crop end date/type and the end of the campaign
+        self.crop_end_type = str(row.crop_end_type).strip().lower()
+        if self.crop_end_type not in ["harvest", "earliest", "maturity"]:
+            msg = ("Unrecognized option for END_TYPE in table "
+                   "CROP_CALENDAR: %s" % row.end_type)
+            raise exc.PCSEError(msg)
+
+        if self.crop_end_type == "maturity":
+            self.crop_end_date = "null"
         else:
-            errstr = "Unknown crop_end_type: not maturity|harvest|earliest!"
-            raise RuntimeError(errstr)
+            self.crop_end_date = check_date(row.crop_end_date)
 
-        # End date of the simulation. This could be different for rotations of crops.
-        timerdata["END_DATE"] = timerdata["CROP_END_DATE"]
+        input = self._build_yaml_agromanagement()
+        self._parse_yaml(input)
 
-    except RuntimeError, exc:
-        errstr = "Failed to retrieve timer data from crop_calendar for "+\
-                 "grid %i, year %i, crop %i: " + str(exc)
-        raise TimerdataError(errstr % (grid, year, crop))
+    def _build_yaml_agromanagement(self):
+        """Builds the YAML agromanagent string"""
+
+        # We do not get a variety_name from the CGMS database, so we make one
+        # as <crop_name>_<grid>_<year>
+        variety_name = "%s_%s_%s" % (self.crop_name, self.grid_no, self.campaign_year)
+        input = self.agro_management_template.format(campaign_start_date=self.campaign_start_date,
+                                                     crop_name=self.crop_name,
+                                                     variety_name=variety_name,
+                                                     crop_start_date=self.crop_start_date,
+                                                     crop_start_type=self.crop_start_type,
+                                                     crop_end_date=self.crop_end_date,
+                                                     crop_end_type=self.crop_end_type,
+                                                     duration=self.max_duration
+                                                     )
+        return input
+
+    def _parse_yaml(self, input):
+        """Parses the input YAML string and assigns to self"""
+        try:
+            items = yaml.safe_load(input)
+        except yaml.YAMLError as e:
+            msg = "Failed parsing agromanagement string %s: %s" % (input, e)
+            raise exc.PCSEError(msg)
+        self.extend(items)
 
-    logger.info("Succesfully retrieved timerdata from crop calendar from database")
-    return timerdata
 
 #-------------------------------------------------------------------------------
 def fetch_sitedata(metadata, grid, year):
     """Retrieve site data from DB for given grid, year.
     
-    Pulls sitedata from the PyWofost database 'SITE' table,
+    Pulls sitedata from the PCSE database 'SITE' table,
     
     Returns a dictionary with site parameter name/value pairs.
     """
 
-    # Define a logger for the PyWofost db_util routines
-    logger = logging.getLogger('PyWofost.db_util')
+    # Define a logger for the PCSE db_util routines
+    logger = logging.getLogger('PCSE.db_util')
 
     try:
         #Get all settings from table 'SITE'
@@ -640,133 +665,21 @@ def fetch_sitedata(metadata, grid, year):
             sitedata['SMLIM']  = float(row.smlim)
         else:
             raise RuntimeError("No rows found")
-    except Exception, e:
+    except Exception as e:
         errstr = "Failed to get site data for year %i and grid %i: " + str(e)
         logger.error(errstr % (year, grid))
         raise SitedataError(errstr % (year, grid))
 
-    logger.info("Succesfully retrieved site variables from database")
+    logger.info("Successfully retrieved site variables from database")
     return sitedata
 
-#----------------------------------------------------------------------------
-class GridWeatherDataProvider(WeatherDataProvider):
-    """Retrieves meteodata from the GRID_WEATHER table in a CGMS database.
-    
-    :param metadata: SqlAlchemy metadata object providing DB access
-    :param grid_no:  Grid ID of PyWofost run
-    :param startdate: Retrieve meteo data starting with startdate
-        (datetime.date object)
-    :param enddate: Retrieve meteo data up to and including enddate
-        (datetime.date object)
-        
-    Note that all meteodata is first retrieved from the DB and stored
-    internally. Therefore, no DB connections are stored within the class
-    instance. This makes that class instances can be pickled.
-    
-    """
-    
-    def __init__(self, metadata, grid_no, startdate, enddate):
-
-        WeatherDataProvider.__init__(self)
-        
-        self.grid_no = grid_no
-        self.startdate = startdate
-        self.enddate = enddate
-        self.timeinterval = (enddate - startdate).days + 1
-        
-        # Get location info (lat/lon/elevation)
-        self._fetch_location_from_db(metadata)
-
-        # Retrieved meteo data
-        self._fetch_grid_weather_from_db(metadata)
-            
-    #---------------------------------------------------------------------------
-    def _fetch_location_from_db(self, metadata):
-        """Retrieves latitude, longitude, elevation from 'grid' table and
-        assigns them to self.latitude, self.longitude, self.elevation."""
-
-        # Pull Latitude value for grid nr from database
-
-        try:
-            table_grid = Table('grid', metadata, autoload=True)
-            r = select([table_grid.c.latitude, table_grid.c.longitude,
-                        table_grid.c.altitude],
-                       table_grid.c.grid_no==self.grid_no).execute()
-            row = r.fetchone()
-            r.close()
-            if row is None:
-                raise Exception
-        except Exception, exc:
-            msg = "Failed deriving location info for grid %s" % self.grid_no
-            raise MeteodataError(msg)
-
-        self.latitude = row.latitude
-        self.longitude = row.longitude
-        self.elevation = row.altitude
-
-        msg = "Succesfully retrieved location information from 'grid' table "+\
-              "for grid %s"
-        self.logger.info(msg % self.grid_no)
-
-    #---------------------------------------------------------------------------
-    def _fetch_grid_weather_from_db(self, metadata):
-        """Retrieves the meteo data from table 'grid_weather'.
-        """
-        
-        try:
-            table_gw = Table('grid_weather', metadata, autoload=True)
-            r = select([table_gw],and_(table_gw.c.grid_no==self.grid_no,
-                                       table_gw.c.day>=self.startdate,
-                                       table_gw.c.day<=self.enddate)
-                       ).execute()
-            rows = r.fetchall()
-
-            c = len(rows)
-            if c < self.timeinterval:
-                msg =  "Only %i records selected from table 'grid_weather' "+\
-                       "for grid %i, period %s -- %s."
-                self.logger.warn(msg % (c, self.grid_no, self.startdate,
-                                        self.enddate))
-
-            meteopackager = self._make_WeatherDataContainer
-            for row in rows:
-                DAY = self.check_keydate(row.day)
-                t = {"DAY": DAY, "LAT": self.latitude,
-                     "LON": self.longitude, "ELEV": self.elevation}
-                wdc = meteopackager(row, t)
-                self._store_WeatherDataContainer(wdc, DAY)
-        except Exception, e:
-            errstr = "Failure reading meteodata: " + str(e)
-            raise MeteodataError(errstr)
-
-        msg = ("Successfully retrieved weather data from 'grid_weather' table "
-               "for grid %s between %s and %s")
-        self.logger.info(msg % (self.grid_no, self.startdate, self.enddate))
-    
-    #---------------------------------------------------------------------------
-    def _make_WeatherDataContainer(self, row, t):
-        """Process record from grid_weather including unit conversion."""
-
-        t.update({"TMAX": float(row.maximum_temperature),
-                  "TMIN": float(row.minimum_temperature),
-                  "VAP":  float(row.vapour_pressure),
-                  "WIND": wind10to2(float(row.windspeed)),
-                  "RAIN": float(row.rainfall)/10.,
-                  "E0":  float(row.e0)/10.,
-                  "ES0": float(row.es0)/10.,
-                  "ET0": float(row.et0)/10.,
-                  "IRRAD": float(row.calculated_radiation)*1000.})
-        wdc = WeatherDataContainer(**t)
-        
-        return wdc
-
 
 #----------------------------------------------------------------------------
 class EnsembleGridWeatherDataProvider(WeatherDataProvider):
     """Retrieves ensemble meteodata from database.
     
     :param metadata: SqlAlchemy metadata object providing DB access
-    :param grid_no:  Grid ID of PyWofost run
+    :param grid_no:  CGMS Grid ID
     :param startdate: Retrieve meteo data starting with startdate
         (datetime.date object)
     :param enddate: Retrieve meteo data up to and including enddate
@@ -819,7 +732,7 @@ class EnsembleGridWeatherDataProvider(WeatherDataProvider):
             r.close()
             if row is None:
                 raise Exception
-        except Exception, exc:
+        except Exception as exc:
             msg = "Failed deriving location info for grid %s" % self.grid_no
             raise MeteodataError(msg)
 
