@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2004-2014 Alterra, Wageningen-UR
-# Allard de Wit (allard.dewit@wur.nl), April 2014
+# Copyright (c) 2004-2018 Alterra, Wageningen-UR
+# Allard de Wit (allard.dewit@wur.nl), April 2018
 """The PCSE Engine provides the environment where SimulationObjects are 'living'.
 The engine takes care of reading the model configuration, initializing model
 components (e.g. groups of SimulationObjects), driving the simulation
@@ -15,26 +15,39 @@ This kind of functionality cannot be implemented in the Engine because
 the model details are not known beforehand.
 """
 import os, sys
-from collections import deque
-import logging
 import datetime
+import gc
 
-from .traitlets import Instance, Bool
-from .base_classes import (VariableKiosk, WeatherDataProvider,
-                           AncillaryObject, WeatherDataContainer,
-                           SimulationObject, BaseEngine)
-from .util import ConfigurationLoader
+from .traitlets import Instance, Bool, List, Dict
+from .base import (VariableKiosk, WeatherDataProvider,
+                           AncillaryObject, SimulationObject,
+                           BaseEngine, ParameterProvider)
+from .util import ConfigurationLoader, check_date
 from .timer import Timer
 from . import signals
 from . import exceptions as exc
 from .settings import settings
 
+
 class Engine(BaseEngine):
     """Simulation engine for simulating the combined soil/crop system.
-    
+
+    :param parameterprovider: A `ParameterProvider` object providing model
+        parameters as key/value pairs. The parameterprovider encapsulates
+        the different parameter sets for crop, soil and site parameters.
+    :param weatherdataprovider: An instance of a WeatherDataProvider that can
+        return weather data in a WeatherDataContainer for a given date.
+    :param agromanagement: AgroManagement data. The data format is described
+        in the section on agronomic management.
+    :param config: A string describing the model configuration file to use.
+        By only giving a filename PCSE assumes it to be located in the 'conf/'
+        folder in the main PCSE folder.
+        If you want to provide you own configuration file, specify
+        it as an absolute or a relative path (e.g. with a leading '.')
+
     `Engine` handles the actual simulation of the combined soil-
     crop system. The central part of the  `Engine` is the soil
-    waterbalance which is continuously simulating during the entire run. In
+    water balance which is continuously simulating during the entire run. In
     contrast, `CropSimulation` objects are only initialized after receiving a
     "CROP_START" signal from the AgroManagement unit. From that point onward,
     the combined soil-crop is simulated including the interactions between
@@ -43,11 +56,11 @@ class Engine(BaseEngine):
     Similarly, the crop simulation is finalized when receiving a "CROP_FINISH"
     signal. At that moment the `finalize()` section on the cropsimulation is
     executed. Moreover, the "CROP_FINISH" signal can specify that the
-    cropsimulation object should be deleted from the hierarchy. The latter is
+    crop simulation object should be deleted from the hierarchy. The latter is
     useful for further extensions of PCSE for running crop rotations.
     
     Finally, the entire simulation is terminated when a "TERMINATE" signal is
-    received. At that point, the `finalize()` section on the waterbalance is 
+    received. At that point, the `finalize()` section on the water balance is
     executed and the simulation stops.
 
     **Signals handled by Engine:**
@@ -72,13 +85,14 @@ class Engine(BaseEngine):
     """
     # system configuration
     mconf = Instance(ConfigurationLoader)
+    parameterprovider = Instance(ParameterProvider)
 
     # sub components for simulation
     crop = Instance(SimulationObject)
     soil = Instance(SimulationObject)
-    agromanagement = Instance(AncillaryObject)
+    agromanager = Instance(AncillaryObject)
     weatherdataprovider = Instance(WeatherDataProvider)
-    drv = Instance(WeatherDataContainer)
+    drv = None
     kiosk = Instance(VariableKiosk)
     timer = Instance(Timer)
     day = Instance(datetime.date)
@@ -92,76 +106,64 @@ class Engine(BaseEngine):
     flag_summary_output = Bool(False)
     
     # placeholders for variables saved during model execution
-    _saved_output = Instance(list)
-    _saved_summary_output = Instance(list)
+    _saved_output = List()
+    _saved_summary_output = List()
+    _saved_terminal_output = Dict()
+
+    def __init__(self, parameterprovider, weatherdataprovider, agromanagement, config=None):
 
-    # Helper variables
-    TMNSAV = Instance(deque)
-    
-    def __init__(self, parameterprovider,
-                 weatherdataprovider, config=None):
-        """
-        :param parameterprovider: A `ParameterProvider` object providing model
-            parameters as key/value pairs. The parameterprovider encapsulates
-            the different parameter files for agromanagement, crop, soil and
-            site.
-        :param weatherdataprovider: An instance of a WeatherDataProvider that can
-            return weather data in a WeatherDataContainer for a given date.
-        :param config: A string describing the model configuration file to use.
-            By only giving a filename PCSE assumes it to be located under
-            pcse/conf. If you want to provide you own configuration file, specify
-            it as an absolute or a relative path (e.g. with a leading '.')
-        """
         BaseEngine.__init__(self)
 
         # Load the model configuration
         self.mconf = ConfigurationLoader(config)
+        self.parameterprovider = parameterprovider
 
         # Variable kiosk for registering and publishing variables
         self.kiosk = VariableKiosk()
 
+        # Placeholder for variables to be saved during a model run
+        self._saved_output = list()
+        self._saved_summary_output = list()
+        self._saved_terminal_output = dict()
+
         # register handlers for starting/finishing the crop simulation, for
         # handling output and terminating the system
         self._connect_signal(self._on_CROP_START, signal=signals.crop_start)
         self._connect_signal(self._on_CROP_FINISH, signal=signals.crop_finish)
         self._connect_signal(self._on_OUTPUT, signal=signals.output)
-        self._connect_signal(self._on_SUMMARY_OUTPUT, signal=signals.summary_output)
         self._connect_signal(self._on_TERMINATE, signal=signals.terminate)
 
+        # Component for agromanagement
+        self.agromanager = self.mconf.AGROMANAGEMENT(self.kiosk, agromanagement)
+        start_date = self.agromanager.start_date
+        end_date = self.agromanager.end_date
+
         # Timer: starting day, final day and model output
-        start_date = parameterprovider["START_DATE"]
-        end_date = parameterprovider["END_DATE"]
-        self.timer = Timer(start_date, self.kiosk, end_date, self.mconf)
-        self.day = self.timer()
+        self.timer = Timer(self.kiosk, start_date, end_date, self.mconf)
+        self.day, delt = self.timer()
 
         # Driving variables
         self.weatherdataprovider = weatherdataprovider
         self.drv = self._get_driving_variables(self.day)
 
         # Component for simulation of soil processes
-        self.soil = self.mconf.SOIL(self.day, self.kiosk, parameterprovider)
+        if self.mconf.SOIL is not None:
+            self.soil = self.mconf.SOIL(self.day, self.kiosk, parameterprovider)
 
-        # Component for agromanagement
-        self.agromanagement = self.mconf.AGROMANAGEMENT(self.day, self.kiosk, self.mconf,
-                                                        parameterprovider)
         # Call AgroManagement module for management actions at initialization
-        self.agromanagement(self.day, self.drv)
-
-        # Placeholder for variables to be saved during a model run
-        self._saved_output = list()
-        self._saved_summary_output = list()
+        self.agromanager(self.day, self.drv)
 
         # Calculate initial rates
         self.calc_rates(self.day, self.drv)
 
-    #---------------------------------------------------------------------------
     def calc_rates(self, day, drv):
 
         # Start rate calculation on individual components
         if self.crop is not None:
             self.crop.calc_rates(day, drv)
 
-        self.soil.calc_rates(day, drv)
+        if self.soil is not None:
+            self.soil.calc_rates(day, drv)
 
         # Save state variables of the model
         if self.flag_output:
@@ -171,16 +173,16 @@ class Engine(BaseEngine):
         if self.flag_crop_finish:
             self._finish_cropsimulation(day)
 
-    #---------------------------------------------------------------------------
-    def integrate(self, day):
+    def integrate(self, day, delt):
 
         # Flush state variables from the kiosk before state updates
         self.kiosk.flush_states()
 
         if self.crop is not None:
-            self.crop.integrate(day)
+            self.crop.integrate(day, delt)
 
-        self.soil.integrate(day)
+        if self.soil is not None:
+            self.soil.integrate(day, delt)
 
         # Set all rate variables to zero
         if settings.ZEROFY:
@@ -189,56 +191,60 @@ class Engine(BaseEngine):
         # Flush rate variables from the kiosk after state updates
         self.kiosk.flush_rates()
 
-    #---------------------------------------------------------------------------
+    def _run(self):
+        """Make one time step of the simulation.
+        """
+
+        # Update timer
+        self.day, delt = self.timer()
+
+        # State integration
+        self.integrate(self.day, delt)
+
+        # Driving variables
+        self.drv = self._get_driving_variables(self.day)
+
+        # Agromanagement decisions
+        self.agromanager(self.day, self.drv)
+
+        # Rate calculation
+        self.calc_rates(self.day, self.drv)
+
+        if self.flag_terminate is True:
+            self._terminate_simulation(self.day)
+
     def run(self, days=1):
         """Advances the system state with given number of days"""
 
         days_done = 0
         while (days_done < days) and (self.flag_terminate is False):
             days_done += 1
+            self._run()
 
-            # Update timer
-            self.day = self.timer()
-
-            # State integration
-            self.integrate(self.day)
-
-            # Driving variables
-            self.drv = self._get_driving_variables(self.day)
-
-            # Agromanagement decisions
-            self.agromanagement(self.day, self.drv)            
-
-            # Rate calculation
-            self.calc_rates(self.day, self.drv)
-        
-        if self.flag_terminate is True:
-            self.soil.finalize(self.day)
-
-    #---------------------------------------------------------------------------
     def run_till_terminate(self):
         """Runs the system until a terminate signal is sent."""
 
         while self.flag_terminate is False:
-            # Update timer
-            self.day = self.timer()
+            self._run()
 
-            # State integration
-            self.integrate(self.day)
+    def run_till(self, rday):
+        """Runs the system until rday is reached."""
 
-            # Driving variables
-            self.drv = self._get_driving_variables(self.day)
+        try:
+            rday = check_date(rday)
+        except KeyError as e:
+            msg = "run_till() function needs a date object as input"
+            print(msg)
+            return
 
-            # Agromanagement decisions
-            self.agromanagement(self.day, self.drv)
+        if rday <= self.day:
+            msg = "date argument for run_till() function before current model date."
+            print(msg)
+            return
 
-            # Rate calculation
-            self.calc_rates(self.day, self.drv)
+        while self.flag_terminate is False and self.day < rday:
+            self._run()
 
-        if self.flag_terminate is True:
-            self.soil.finalize(self.day)
-
-    #---------------------------------------------------------------------------
     def _on_CROP_FINISH(self, day, crop_delete=False):
         """Sets the variable 'flag_crop_finish' to True when the signal
         CROP_FINISH is received.
@@ -255,47 +261,36 @@ class Engine(BaseEngine):
         """
         self.flag_crop_finish = True
         self.flag_crop_delete = crop_delete
-        self.flag_summary_output = True
-        
-    #---------------------------------------------------------------------------
-    def _on_CROP_START(self, day, cropsimulation):
-        """Receives a crop simulation object from the agromanagement unit.
-        This object is than assigned to self.cropsimulation and will used
-        for simulation of crop dynamics.
 
+    def _on_CROP_START(self, day, crop_name=None, variety_name=None,
+                       crop_start_type=None, crop_end_type=None):
+        """Starts the crop
         """
         self.logger.debug("Received signal 'CROP_START' on day %s" % day)
 
         if self.crop is not None:
-            msg = ("A CROP_START signal was received while self.cropsimulation "+
-                   "still holds a valid cropsimulation object. It looks like " +
-                   "you forgot to send a CROP_FINISH signal with option" +
+            msg = ("A CROP_START signal was received while self.cropsimulation "
+                   "still holds a valid cropsimulation object. It looks like "
+                   "you forgot to send a CROP_FINISH signal with option "
                    "crop_delete=True")
             raise exc.PCSEError(msg)
-        self.crop = cropsimulation
-    
-    #---------------------------------------------------------------------------
+
+        self.parameterprovider.set_active_crop(crop_name, variety_name, crop_start_type,
+                                               crop_end_type)
+        self.crop = self.mconf.CROP(day, self.kiosk, self.parameterprovider)
+
     def _on_TERMINATE(self):
         """Sets the variable 'flag_terminate' to True when the signal TERMINATE
         was received.
         """
         self.flag_terminate = True
         
-    #---------------------------------------------------------------------------
     def _on_OUTPUT(self):
         """Sets the variable 'flag_output to True' when the signal OUTPUT
         was received.
         """
         self.flag_output = True
         
-    #---------------------------------------------------------------------------
-    def _on_SUMMARY_OUTPUT(self):
-        """Sets the variable 'flag_summary_output to True' when the signal
-        SUMMARY_OUTPUT was received.
-        """
-        self.flag_summary_output = True
-
-    #---------------------------------------------------------------------------
     def _finish_cropsimulation(self, day):
         """Finishes the CropSimulation object when variable 'flag_crop_finish'
         has been set to True based on the signal 'CROP_FINISH' being
@@ -307,8 +302,11 @@ class Engine(BaseEngine):
         self.crop.finalize(day)
 
         # Generate summary output after finalize() has been run.
-        if self.flag_summary_output:
-            self._save_summary_output()
+        self._save_summary_output()
+
+        # Clear any override parameters in the ParameterProvider to avoid
+        # lagging parameters for the next crop
+        self.parameterprovider.clear_override()
 
         # Only remove the crop simulation object from the system when the crop
         # is finished, when explicitly asked to do so.
@@ -316,8 +314,23 @@ class Engine(BaseEngine):
             self.flag_crop_delete = False
             self.crop._delete()
             self.crop = None
+            # Run a dedicated garbage collection, because it was demonstrated
+            # that the standard python GC did not garbage collect the crop
+            # simulation object. This caused signals to be received by crop simulation
+            # objects that were supposed to be garbage collected already.
+            gc.collect()
+
+    def _terminate_simulation(self, day):
+        """Terminates the entire simulation.
+
+        First the finalize() call on the soil component is executed.
+        Next, the TERMINAL_OUTPUT is collected and stored.
+        """
+
+        if self.soil is not None:
+            self.soil.finalize(self.day)
+        self._save_terminal_output()
 
-    #---------------------------------------------------------------------------
     def _get_driving_variables(self, day):
         """Get driving variables, compute derived properties and return it.
         """
@@ -329,26 +342,8 @@ class Engine(BaseEngine):
         if not hasattr(drv, "DTEMP"):
             drv.add_variable("DTEMP", (drv.TEMP + drv.TMAX)/2., "Celcius")
 
-        #  7 day running average of minimum temperature
-        TMINRA = self._7day_running_avg(drv.TMIN)
-        if not hasattr(drv, "TMINRA"):
-            drv.add_variable("TMINRA", TMINRA, "Celsius")
-        
         return drv
 
-    #---------------------------------------------------------------------------
-    def _7day_running_avg(self, TMIN):
-        """Calculate 7-day running mean of minimum temperature.
-        """
-        # if self.TMNSAV is None, then initialize a deque of size 7
-        if self.TMNSAV is None:
-            self.TMNSAV = deque(maxlen=7)
-        # Append new value
-        self.TMNSAV.appendleft(TMIN)
-
-        return sum(self.TMNSAV)/len(self.TMNSAV)
-    
-    #---------------------------------------------------------------------------
     def _save_output(self, day):
         """Appends selected model variables to self._saved_output for this day.
         """
@@ -361,20 +356,22 @@ class Engine(BaseEngine):
             states[var] = self.get_variable(var)
         self._saved_output.append(states)
 
-    #---------------------------------------------------------------------------
     def _save_summary_output(self):
         """Appends selected model variables to self._saved_summary_output.
         """
-        # Switch off the flag for generating output
-        self.flag_summary_output = False
-
         # find current value of variables to are to be saved
         states = {}
         for var in self.mconf.SUMMARY_OUTPUT_VARS:
             states[var] = self.get_variable(var)
         self._saved_summary_output.append(states)
 
-    #---------------------------------------------------------------------------
+    def _save_terminal_output(self):
+        """Appends selected model variables to self._saved_terminal_output.
+        """
+        # find current value of variables to are to be saved
+        for var in self.mconf.TERMINAL_OUTPUT_VARS:
+            self._saved_terminal_output[var] = self.get_variable(var)
+
     def set_variable(self, varname, value):
         """Sets the value of the specified state or rate variable.
 
@@ -403,14 +400,145 @@ class Engine(BaseEngine):
         with the value '5.0' as input.
         """
         increments = {}
-        self.soil.set_variable(varname, value, increments)
+        if self.soil is not None:
+            self.soil.set_variable(varname, value, increments)
         if self.crop is not None:
             self.crop.set_variable(varname, value, increments)
 
         return increments
 
     def get_output(self):
+        """Returns the variables have have been stored during the simulation.
+
+        If no output is stored an empty list is returned. Otherwise, the output is
+        returned as a list of dictionaries in chronological order. Each dictionary is
+        a set of stored model variables for a certain date. """
+
         return self._saved_output
 
     def get_summary_output(self):
+        """Returns the summary variables have have been stored during the simulation.
+        """
+
         return self._saved_summary_output
+
+    def get_terminal_output(self):
+        """Returns the terminal output variables have have been stored during the simulation.
+        """
+
+        return self._saved_terminal_output
+
+
+class CGMSEngine(Engine):
+    """Engine to mimic CGMS behaviour.
+
+    The original CGMS did not terminate when the crop cycles was finished but instead continued with its
+    simulation cycle but without altering the crop and soil components. This had the effect that after the
+    crop cycle finished, all state variables were kept at the same value while the day counter increased.
+    This behaviour is useful for two reasons:
+
+    1. CGMS generally produces dekadal output and when a day-of-maturity or day-of-harvest does not coincide
+       with a dekad boundary the final simulation values remain available and are stored at the next dekad.
+    2. When aggregating spatial simulations with variability in day-of-maturity or day-of-harvest it ensures
+       that records are available in the database tables. So GroupBy clauses in SQL queries produce the right
+       results when computing spatial averages.
+
+    The difference with the Engine are:
+
+    1. Crop rotations are not supported
+    2. After a CROP_FINISH signal, the engine will continue, updating the
+       timer but the soil, crop and agromanagement will not execute their simulation cycles.
+       As a consequence, all state variables will retain their value.
+    3. TERMINATE signals have no effect.
+    4. CROP_FINISH signals will never remove the CROP SimulationObject.
+    5. run() and run_till_terminate() are not supported, only run_till() is supported.
+    """
+
+    flag_crop_finish = False
+    # Because of the intended behaviour of CGMSEngine flag_crop_delete is always False
+    flag_crop_delete = False
+
+    def run(self, days=1):
+        msg = "run() is not supported in the CGMSEngine, use: run_till(<date>)"
+        raise NotImplementedError(msg)
+
+    def run_till_terminate(self):
+        msg = "run_till_terminate() is not supported in the CGMSEngine, use: run_till(<date>)"
+        raise NotImplementedError(msg)
+
+    def run_till(self, rday):
+        """Runs the system until rday is reached."""
+
+        try:
+            rday = check_date(rday)
+        except KeyError as e:
+            msg = "run_till() function needs a date object as input"
+            print(msg)
+            return
+
+        if rday <= self.day:
+            msg = "date argument for run_till() function before current model date."
+            print(msg)
+            return
+
+        while self.day < rday:
+            self._run()
+
+    def _run(self):
+        """Make one time step of the simulation.
+        """
+
+        # Update timer
+        self.day, delt = self.timer()
+
+        if self.flag_crop_finish is False:
+            # State integration
+            self.integrate(self.day, delt)
+
+            # Driving variables
+            self.drv = self._get_driving_variables(self.day)
+
+            # Agromanagement decisions
+            self.agromanager(self.day, self.drv)
+
+            # Rate calculation
+            self.calc_rates(self.day, self.drv)
+
+        elif self.flag_crop_finish is True:
+            # Run the finalize section of the crop and soil simulation and sub-components
+            if self.crop is not None:
+                self.crop.finalize(self.day)
+            if self.soil is not None:
+                self.soil.finalize(self.day)
+
+            # Generate summary output after finalize() has been run.
+            self._save_summary_output()
+
+            # Set self.flag_crop_finish to None indicating that the simulation cycle should not continue
+            self.flag_crop_finish = None
+
+            # Still retain output if flag is set
+            if self.flag_output:
+                self._save_output(self.day)
+        else:
+            # Do nothing but still retain output if flag is set
+            if self.flag_output:
+                self._save_output(self.day)
+
+    def _on_CROP_FINISH(self, day, *args, **kwargs):
+        """Sets the variable 'flag_crop_finish' to True when the signal
+        CROP_FINISH is received.
+
+        """
+        self.flag_crop_finish = True
+
+    def _on_TERMINATE(self):
+        "TERMINATE is not implemented for CGMS Engine. This is only here to intercept the TERMINATE signal."
+        pass
+
+    def _finish_cropsimulation(self, day):
+        """This is already implemented in _run().
+
+        This method is just here to intercept the call to _finish_cropsimulation() from calc_rates().
+        """
+        pass
\ No newline at end of file
