@@ -4,7 +4,7 @@
 """Tools for reading  weather and parameter files.
 
 For reading files in the CABO formats used by crop simulation models in FORTRAN and FST:
-- CABOWeatherDataProvider reads CABOWE weather files for use in PyWOFOST
+- CABOWeatherDataProvider reads CABOWE weather files for use in PCSE
 - CABOFileReader reads CABO parameter files.
 
 For reading the new PCSE format use:
@@ -14,4 +14,8 @@ For reading the new PCSE format use:
 
 from .cabo_reader import CABOFileReader
 from .cabo_weather import CABOWeatherDataProvider
-from .pcsefilereader import PCSEFileReader
\ No newline at end of file
+from .pcsefilereader import PCSEFileReader
+from .xlsweatherdataprovider import ExcelWeatherDataProvider
+from .yaml_agro_loader import YAMLAgroManagementReader
+from .csvweatherdataprovider import CSVWeatherDataProvider
+from .yaml_cropdataprovider import YAMLCropDataProvider
\ No newline at end of file
