@@ -3,14 +3,14 @@
 # Allard de Wit (allard.dewit@wur.nl), April 2014
 """Python implementations of the WOFOST waterbalance modules for simulation
 of potential production (`WaterbalancePP`) and water-limited production
-(`WaterbalanceFD`)under freely draining conditions.
+(`WaterbalanceFD`) under freely draining conditions.
 """
 from math import sqrt
 
-from ..traitlets import Float, Int, Instance, Enum, Unicode, Bool
+from ..traitlets import Float, Int, Instance, Enum, Unicode, Bool, List
 from ..decorators import prepare_rates, prepare_states
 from ..util import limit, Afgen, merge_dict
-from ..base_classes import ParamTemplate, StatesTemplate, RatesTemplate, \
+from ..base import ParamTemplate, StatesTemplate, RatesTemplate, \
      SimulationObject
 from .. import signals
 from .. import exceptions as exc
@@ -19,35 +19,83 @@ from .snowmaus import SnowMAUS
 
 class WaterbalancePP(SimulationObject):
     """Fake waterbalance for simulation under potential production.
+
+    Keeps the soil moisture content at field capacity and only accumulates crop transpiration
+    and soil evaporation rates through the course of the simulation
     """
-    
+    # Counter for Days-Dince-Last-Rain
+    DSLR = Float(1)
+    # rainfall rate of previous day
+    RAINold = Float(0)
+
     class Parameters(ParamTemplate):
         SMFCF = Float(-99.)
 
     class StateVariables(StatesTemplate):
         SM = Float(-99.)
+        WTRAT = Float(-99.)
+        EVST = Float(-99.)
+
+    class RateVariables(RatesTemplate):
+        EVS = Float(-99.)
+        WTRA = Float(-99.)
 
     def initialize(self, day, kiosk, parvalues):
         """    
         :param day: start date of the simulation
         :param kiosk: variable kiosk of this PCSE  instance
-        :param soildata: dictionary with WOFOST soildata key/value pairs
+        :param parvalues: ParameterProvider object containing all parameters
     
         This waterbalance keeps the soil moisture always at field capacity. Therefore   
         `WaterbalancePP` has only one parameter (`SMFCF`: the field capacity of the
         soil) and one state variable (`SM`: the volumetric soil moisture).
         """
         self.params = self.Parameters(parvalues)
+        self.rates = self.RateVariables(kiosk, publish="EVS")
         self.states = self.StateVariables(kiosk, SM=self.params.SMFCF,
-                                          publish="SM")
+                                          publish="SM", EVST=0, WTRAT=0)
     
     @prepare_rates
     def calc_rates(self, day, drv):
-        pass
-    
+
+        r = self.rates
+        # Transpiration and maximum soil and surface water evaporation rates
+        # are calculated by the crop Evapotranspiration module.
+        # However, if the crop is not yet emerged then set TRA=0 and use
+        # the potential soil/water evaporation rates directly because there is
+        # no shading by the canopy.
+        if "TRA" not in self.kiosk:
+            r.WTRA = 0.
+            EVSMX = drv.ES0
+        else:
+            r.WTRA = self.kiosk["TRA"]
+            EVSMX = self.kiosk["EVSMX"]
+
+        # Actual evaporation rates
+
+        if self.RAINold >= 1:
+            # If rainfall amount >= 1cm on previous day assume maximum soil
+            # evaporation
+            r.EVS = EVSMX
+            self.DSLR = 1.
+        else:
+            # Else soil evaporation is a function days-since-last-rain (DSLR)
+            self.DSLR += 1
+            EVSMXT = EVSMX * (sqrt(self.DSLR) - sqrt(self.DSLR - 1))
+            r.EVS = min(EVSMX, EVSMXT + self.RAINold)
+
+        # Hold rainfall amount to keep track of soil surface wetness and reset self.DSLR if needed
+        self.RAINold = drv.RAIN
+
     @prepare_states
-    def integrate(self, day):
+    def integrate(self, day, delt=1.0):
+
+        # Keep soil moisture on field capacity
         self.states.SM = self.params.SMFCF
+
+        # Accumulated transpiration and soil evaporation amounts
+        self.states.EVST += self.rates.EVS * delt
+        self.states.WTRAT += self.rates.WTRA * delt
         
 
 class WaterbalanceFD(SimulationObject):
@@ -66,15 +114,21 @@ class WaterbalanceFD(SimulationObject):
     zone to second layer), and the loss of water beyond the maximum root zone. 
 
     The textural profile of the soil is conceived as homogeneous. Initially the
-    soil profile consists of two layers, the actually rooted  soil and the soil
-    immediately below the rooted zone until the maximum rooting depth (soil and
-    crop dependent). The extension of the root zone from initial rooting depth
-    to maximum rooting depth is described in Root_Dynamics class. From the
-    moment that the maximum rooting depth is reached the soil profile is
-    described as a one layer system. The class WaterbalanceFD is derived
-    from WATFD.FOR in WOFOST7.1
+    soil profile consists of two layers, the actually rooted soil and the soil
+    immediately below the rooted zone until the maximum rooting depth is reached
+    by roots(soil and crop dependent). The extension of the root zone from the
+    initial rooting depth to maximum rooting depth is described in Root_Dynamics
+    class. From the moment that the maximum rooting depth is reached the soil
+    profile may be described as a one layer system depending if the roots are
+    able to penetrate the entire profile. If not a non-rooted part remains
+    at the bottom of the profile.
+
+    The class WaterbalanceFD is derived from WATFD.FOR in WOFOST7.1 with the
+    exception that the depth of the soil is now completely determined by the
+    maximum soil depth (RDMSOL) and not by the minimum of soil depth and crop
+    maximum rooting depth (RDMCR).
     
-    **Simulation parameters:** (provide in crop, soil and sitedata dictionary)
+    **Simulation parameters:**
     
     ======== =============================================== =======  ==========
      Name     Description                                     Type     Unit
@@ -85,7 +139,6 @@ class WaterbalanceFD(SimulationObject):
     CRAIRC    Soil critical air content (waterlogging)         SSo     -
     SOPE      maximum percolation rate root zone               SSo    |cmday-1|
     KSUB      maximum percolation rate subsoil                 SSo    |cmday-1|
-    K0        hydraulic conductivity of saturated soil         SSo    |cmday-1|
     RDMSOL    Soil rootable depth                              SSo     cm
     IFUNRN    Indicates whether non-infiltrating fraction of   SSi    -
               rain is a function of storm size (1)
@@ -98,9 +151,6 @@ class WaterbalanceFD(SimulationObject):
               the soil
     SMLIM     Initial maximum moisture content in initial      SSi     -
               rooting depth zone.
-    IAIRDU    Switch airducts on (1) or off (0)                SCr     - 
-    RDMCR     Maximum rooting depth of the crop                SCr      cm
-    RDI       Initial rooting depth of the crop                SCr      cm
     ======== =============================================== =======  ==========
 
     **State variables:**
@@ -110,6 +160,7 @@ class WaterbalanceFD(SimulationObject):
     =======  ================================================= ==== ============
     SM        Volumetric moisture content in root zone          Y    -
     SS        Surface storage (layer of water on surface)       N    cm
+    SSI       Initial urface storage                            N    cm
     W         Amount of water in root zone                      N    cm
     WI        Initial amount of water in the root zone          N    cm
     WLOW      Amount of water in the subsoil (between current   N    cm
@@ -124,15 +175,16 @@ class WaterbalanceFD(SimulationObject):
     EVST      Total evaporation from the soil surface           N    cm
     EVWT      Total evaporation from a water surface            N    cm
     TSR       Total surface runoff                              N    cm
-    RAINT     Total amount of rainfall                          N    cm
+    RAINT     Total amount of rainfall (eff + non-eff)          N    cm
     WDRT      Amount of water added to root zone by increase    N    cm
               of root growth
     TOTINF    Total amount of infiltration                      N    cm
-    TOTIRR    Total amount of irrigation (not implemented       N    cm
-              yet)
+    TOTIRR    Total amount of effective irrigation              N    cm
     PERCT     Total amount of water percolating from rooted     N    cm
               zone to subsoil
     LOSST     Total amount of water lost to deeper soil         N    cm
+    DSOS      Days since oxygen stress, accumulates the number  Y     -
+              of consecutive days of oxygen stress
     WBALRT    Checksum for root zone waterbalance. Will be      N    cm
               calculated within `finalize()`, abs(WBALRT) >
               0.0001 will raise a WaterBalanceError
@@ -143,25 +195,28 @@ class WaterbalanceFD(SimulationObject):
 
     **Rate variables:**
 
-    ======== ================================================= ==== ============
-     Name     Description                                      Pbl      Unit
-    ======== ================================================= ==== ============
-    EVS      Actual evaporation rate from soil                  N    |cmday-1|
-    EVW      Actual evaporation rate from water surface         N    |cmday-1|
-    WTRA     Actual transpiration rate from plant canopy,       N    |cmday-1|
-             is directly derived from the variable "TRA" in
-             the evapotranspiration module 
-    RAIN     Rainfall rate for current day                      N    |cmday-1|
-    RIN      Infiltration rate for current day                  N    |cmday-1|
-    RIRR     Irrigation rate for current day. Is always set     N    |cmday-1|
-             to zero as irrigation is not handled currently
-    PERC     Percolation rate to non-rooted zone                N    |cmday-1|
-    LOSS     Rate of water loss to deeper soil                  N    |cmday-1|
-    DW       Change in amount of water in rooted zone as a      N    |cmday-1|
-             result of infiltration, transpiration and
-             evaporation.
-    DWLOW    Change in amount of water in subsoil               N    |cmday-1|
-    ======== ================================================= ==== ============
+    =========== ================================================= ==== ============
+     Name        Description                                      Pbl      Unit
+    =========== ================================================= ==== ============
+    EVS         Actual evaporation rate from soil                  N    |cmday-1|
+    EVW         Actual evaporation rate from water surface         N    |cmday-1|
+    WTRA        Actual transpiration rate from plant canopy,       N    |cmday-1|
+                is directly derived from the variable "TRA" in
+                the evapotranspiration module
+    RAIN_INF    Infiltrating rainfall rate for current day         N    |cmday-1|
+    RAIN_NOTINF Non-infiltrating rainfall rate for current day   N    |cmday-1|
+    RIN         Infiltration rate for current day                  N    |cmday-1|
+    RIRR        Effective irrigation rate for current day,         N    |cmday-1|
+                computed as irrigation amount * efficiency.
+    PERC        Percolation rate to non-rooted zone                N    |cmday-1|
+    LOSS        Rate of water loss to deeper soil                  N    |cmday-1|
+    DW          Change in amount of water in rooted zone as a      N    |cmday-1|
+                result of infiltration, transpiration and
+                evaporation.
+    DWLOW       Change in amount of water in subsoil               N    |cmday-1|
+    DTSR        Change in surface runoff                           N    |cmday-1|
+    DSS         Change in surface storage                          N    |cmday-1|
+    =========== ================================================= ==== ============
     
     
     **External dependencies:**
@@ -195,9 +250,16 @@ class WaterbalanceFD(SimulationObject):
     NINFTB = Instance(Afgen)
     # Flag indicating crop present or not
     in_crop_cycle = Bool(False)
-    # Flag indicating that a crop was removed and therefore the thickness 
-    # of the rootzone shift back to its initial value (params.RDI)
+    # Flag indicating that a crop was started or finished and therefore the depth
+    # of the root zone may have changed, required a redistribution of water
+    # between the root zone and the lower zone
     rooted_layer_needs_reset = Bool(False)
+    # placeholder for irrigation
+    _RIRR = Float(0.)
+    # default depth of upper layer (root zone depth)
+    DEFAULT_RD = Float(10.)
+    # Increments on WLOW due to state updates
+    _increments_W = List()
 
     class Parameters(ParamTemplate):
         # Soil parameters
@@ -207,7 +269,6 @@ class WaterbalanceFD(SimulationObject):
         CRAIRC = Float(-99.)
         SOPE   = Float(-99.)
         KSUB   = Float(-99.)
-        K0     = Float(-99.)
         RDMSOL = Float(-99.)
         # Site parameters
         IFUNRN = Float(-99.)
@@ -215,14 +276,11 @@ class WaterbalanceFD(SimulationObject):
         SSI    = Float(-99.)
         WAV    = Float(-99.)
         NOTINF = Float(-99.)
-        # crop parameters
-        IAIRDU = Float(-99.)
-        RDMCR  = Float(-99.)
-        RDI    = Float(-99.)
 
     class StateVariables(StatesTemplate):
         SM = Float(-99.)
         SS = Float(-99.)
+        SSI = Float(-99.)
         W  = Float(-99.)
         WI = Float(-99.)
         WLOW  = Float(-99.)
@@ -242,34 +300,31 @@ class WaterbalanceFD(SimulationObject):
         # Checksums for rootzone (RT) and total system (TT)
         WBALRT = Float(-99.)
         WBALTT = Float(-99.)
+        DSOS = Int(-99)
 
     class RateVariables(RatesTemplate):
         EVS   = Float(-99.)
         EVW   = Float(-99.)
         WTRA  = Float(-99.)
-        RAIN  = Float(-99.)
         RIN   = Float(-99.)
         RIRR  = Float(-99.)
         PERC  = Float(-99.)
         LOSS  = Float(-99.)
         DW    = Float(-99.)
         DWLOW = Float(-99.)
+        DTSR = Float(-99.)
+        DSS = Float(-99.)
+        DRAINT = Float(-99.)
 
     def initialize(self, day, kiosk, parvalues):
         """
         :param day: start date of the simulation
         :param kiosk: variable kiosk of this PCSE  instance
-        :param cropdata: dictionary with WOFOST cropdata key/value pairs
-        :param soildata: dictionary with WOFOST soildata key/value pairs
-        :param sitedata: dictionary with WOFOST sitedata key/value pairs
+        :param parvalues: ParameterProvider containing all parameters
         """
 
         # Check validity of maximum soil moisture amount in topsoil (SMLIM)
-        SMLIM = parvalues["SMLIM"]
-        if parvalues["IAIRDU"] == 1: # applicable only for flooded rice crops
-            SMLIM = parvalues["SM0"]
-        else:
-            SMLIM = limit(parvalues["SMW"], parvalues["SM0"], SMLIM)
+        SMLIM = limit(parvalues["SMW"], parvalues["SM0"],  parvalues["SMLIM"])
 
         if SMLIM != parvalues["SMLIM"]:
             msg = "SMLIM not in valid range, changed from %f to %f."
@@ -279,9 +334,9 @@ class WaterbalanceFD(SimulationObject):
         self.params = self.Parameters(parvalues)
         p = self.params
         
-        # Current, maximum and old rooting depth
-        RD = p.RDI
-        RDM = max(p.RDI, min(p.RDMSOL, p.RDMCR))
+        # set default RD to 10 cm, also derive maximum depth and old rooting depth
+        RD = self.DEFAULT_RD
+        RDM = max(RD, p.RDMSOL)
         self.RDold = RD
         self.RDM = RDM
         
@@ -312,32 +367,35 @@ class WaterbalanceFD(SimulationObject):
         self.NINFTB = Afgen([0.0,0.0, 0.5,0.0, 1.5,1.0])
 
         # Initialize model state variables.       
-        self.states = self.StateVariables(kiosk, publish="SM", SM=SM, SS=SS,
-                           W=W, WI=WI, WLOW=WLOW, WLOWI=WLOWI, WWLOW=WWLOW,
-                           WTRAT=0., EVST=0., EVWT=0., TSR=0.,
-                           RAINT=0., WDRT=0., TOTINF=0., TOTIRR=0.,
+        self.states = self.StateVariables(kiosk, publish=["SM", "DSOS"], SM=SM, SS=SS,
+                           SSI=p.SSI, W=W, WI=WI, WLOW=WLOW, WLOWI=WLOWI,
+                           WWLOW=WWLOW, WTRAT=0., EVST=0., EVWT=0., TSR=0.,
+                           RAINT=0., WDRT=0., TOTINF=0., TOTIRR=0., DSOS=0,
                            PERCT=0., LOSST=0., WBALRT=-999., WBALTT=-999.)
-        self.rates = self.RateVariables(kiosk)
+        self.rates = self.RateVariables(kiosk, publish="EVS")
         
-        # Connect to CROP_EMERGED/CROP_FINISH signals for water balance to
+        # Connect to CROP_START/CROP_FINISH signals for water balance to
         # search for crop transpiration values
         self._connect_signal(self._on_CROP_START, signals.crop_start)
         self._connect_signal(self._on_CROP_FINISH, signals.crop_finish)
-        
+        # signal for irrigation
+        self._connect_signal(self._on_IRRIGATE, signals.irrigate)
+
+        self._increments_W = []
+
     @prepare_rates
     def calc_rates(self, day, drv):
         s = self.states
         p = self.params
         r = self.rates
+        k = self.kiosk
 
-        # Rate of irrigation (RIRR) set to zero
-        r.RIRR = 0.
-        
-        # Rainfall rate
-        r.RAIN = drv.RAIN
+        # Rate of irrigation (RIRR)
+        r.RIRR = self._RIRR
+        self._RIRR = 0.
 
         # Transpiration and maximum soil and surface water evaporation rates
-        # are calculated by the crop Evapotranspiration module. 
+        # are calculated by the crop evapotranspiration module.
         # However, if the crop is not yet emerged then set TRA=0 and use
         # the potential soil/water evaporation rates directly because there is
         # no shading by the canopy.
@@ -346,17 +404,17 @@ class WaterbalanceFD(SimulationObject):
             EVWMX = drv.E0
             EVSMX = drv.ES0
         else:
-            r.WTRA = self.kiosk["TRA"]
-            EVWMX = self.kiosk["EVWMX"]
-            EVSMX = self.kiosk["EVSMX"]
-        
+            r.WTRA = k.TRA
+            EVWMX = k.EVWMX
+            EVSMX = k.EVSMX
+
         # Actual evaporation rates
         r.EVW = 0.
         r.EVS = 0.
         if s.SS > 1.:
             # If surface storage > 1cm then evaporate from water layer on
             # soil surface
-            r.EVS = EVSMX
+            r.EVW = EVWMX
         else:
             # else assume evaporation from soil surface
             if self.RINold >= 1:
@@ -366,22 +424,24 @@ class WaterbalanceFD(SimulationObject):
                 self.DSLR = 1.
             else:
                 # Else soil evaporation is a function days-since-last-rain (DSLR)
-                self.DSLR += 1
-                EVSMXT = EVSMX*(sqrt(self.DSLR) - sqrt(self.DSLR-1))
+                EVSMXT = EVSMX * (sqrt(self.DSLR + 1) - sqrt(self.DSLR))
                 r.EVS = min(EVSMX, EVSMXT + self.RINold)
-        
-        # Preliminary infiltration rate (RINPRE)
-        if s.SS < 0.1:
-            # without surface storage
-            if (p.IFUNRN == 0):
-                RINPRE = (1. - p.NOTINF)*drv.RAIN + r.RIRR + s.SS
+                self.DSLR += 1
 
-            else:
-                RINPRE = (1. - p.NOTINF*self.NINFTB(drv.RAIN)) * drv.RAIN + \
-                         r.RIRR + s.SS
+        # Potentially infiltrating rainfall
+        if p.IFUNRN == 0:
+            RINPRE = (1. - p.NOTINF) * drv.RAIN
         else:
+            # infiltration is function of storm size (NINFTB)
+            RINPRE = (1. - p.NOTINF * self.NINFTB(drv.RAIN)) * drv.RAIN
+
+
+        # Second stage preliminary infiltration rate (RINPRE)
+        # including surface storage and irrigation
+        RINPRE = RINPRE + r.RIRR + s.SS
+        if s.SS > 0.1:
             # with surface storage, infiltration limited by SOPE
-            AVAIL = s.SS + (drv.RAIN * (1.-p.NOTINF)) + r.RIRR - r.EVW
+            AVAIL = RINPRE + r.RIRR - r.EVW
             RINPRE = min(p.SOPE, AVAIL)
             
         RD = self._determine_rooting_depth()
@@ -396,27 +456,42 @@ class WaterbalanceFD(SimulationObject):
         # loss of water at the lower end of the maximum root zone
         # equilibrium amount of soil moisture below rooted zone
         WELOW = p.SMFCF * (self.RDM - RD)
-        LOSS  = limit(0., p.KSUB, (s.WLOW - WELOW + PERC1))
-        # for rice water losses are limited to K0/20
-        if (p.IAIRDU == 1):
-            LOSS = min(LOSS, p.K0/20.)
-        r.LOSS = LOSS
+        r.LOSS = limit(0., p.KSUB, (s.WLOW - WELOW + PERC1))
 
         # percolation not to exceed uptake capacity of subsoil
-        PERC2 = ((self.RDM -RD) * p.SM0 - s.WLOW) + LOSS
-        r.PERC  = min(PERC1,PERC2)
+        PERC2 = ((self.RDM - RD) * p.SM0 - s.WLOW) + r.LOSS
+        r.PERC = min(PERC1, PERC2)
 
         # adjustment of infiltration rate
         r.RIN = min(RINPRE, (p.SM0 - s.SM)*RD + r.WTRA + r.EVS + r.PERC)
         self.RINold = r.RIN
 
         # rates of change in amounts of moisture W and WLOW
-        r.DW    = r.RIN - r.WTRA - r.EVS - r.PERC
+        r.DW = r.RIN - r.WTRA - r.EVS - r.PERC
         r.DWLOW = r.PERC - r.LOSS
 
+        # Check if DW creates a negative value of W
+        # If so, reduce EVS to reach W == 0
+        Wtmp = s.W + r.DW
+        if Wtmp < 0.0:
+            r.EVS += Wtmp
+            assert r.EVS >= 0., "Negative soil evaporation rate on day %s: %s" % (day, r.EVS)
+            r.DW = -s.W
+
+        # Computation of rate of change in surface storage and surface runoff
+        # SStmp is the layer of water that cannot infiltrate and that can potentially
+        # be stored on the surface. Here we assume that RAIN_NOTINF automatically
+        # ends up in the surface storage (and finally runoff).
+        SStmp = drv.RAIN + r.RIRR - r.EVW - r.RIN
+        # rate of change in surface storage is limited by SSMAX - SS
+        r.DSS = min(SStmp, (p.SSMAX - s.SS))
+        # Remaining part of SStmp is send to surface runoff
+        r.DTSR = SStmp - r.DSS
+        # incoming rainfall rate
+        r.DRAINT = drv.RAIN
 
     @prepare_states
-    def integrate(self, day):
+    def integrate(self, day, delt=1.0):
         s = self.states
         p = self.params
         r = self.rates
@@ -424,69 +499,50 @@ class WaterbalanceFD(SimulationObject):
         # INTEGRALS OF THE WATERBALANCE: SUMMATIONS AND STATE VARIABLES
 
         # total transpiration
-        s.WTRAT += r.WTRA
+        s.WTRAT += r.WTRA * delt
 
         # total evaporation from surface water layer and/or soil
-        s.EVWT += r.EVW
-        s.EVST += r.EVS
+        s.EVWT += r.EVW * delt
+        s.EVST += r.EVS * delt
 
         # totals for rainfall, irrigation and infiltration
-        s.RAINT  += r.RAIN
-        s.TOTINF += r.RIN
-        s.TOTIRR += r.RIRR
+        s.RAINT += r.DRAINT * delt
+        s.TOTINF += r.RIN * delt
+        s.TOTIRR += r.RIRR * delt
 
-        # Update surface storage, any storage > SSMAX goes to total surface
-        # runoff (TSR)
-        SSPRE = s.SS + (r.RAIN + r.RIRR -r.EVW - r.RIN)
-        s.SS  = min(SSPRE, p.SSMAX)
-        s.TSR += (SSPRE - s.SS)
+        # Update surface storage and total surface runoff (TSR)
+        s.SS += r.DSS * delt
+        s.TSR += r.DTSR * delt
 
         # amount of water in rooted zone
-        W_NEW = s.W + r.DW
-        if (W_NEW < 0.0):
-            # If negative soil water depth, set W to zero and subtract W_NEW
-            # from total soil evaporation to keep the balance. 
-            # Note: W_NEW is negative here!!
-            s.EVST += W_NEW
-            s.W = 0.0
-        else:
-            s.W = W_NEW
+        s.W += r.DW * delt
+        assert s.W >= 0., "Negative amount of water in root zone on day %s: %s" % (day, s.W)
 
         # total percolation and loss of water by deep leaching
-        s.PERCT += r.PERC
-        s.LOSST += r.LOSS
+        s.PERCT += r.PERC * delt
+        s.LOSST += r.LOSS * delt
 
         # amount of water in unrooted, lower part of rootable zone
-        s.WLOW += r.DWLOW
+        s.WLOW += r.DWLOW * delt
         # total amount of water in the whole rootable zone
-        s.WWLOW = s.W + s.WLOW
+        s.WWLOW = s.W + s.WLOW * delt
 
         # CHANGE OF ROOTZONE SUBSYSTEM BOUNDARY
 
-        # Redefine the rootzone when the crop is finished. As a result there
-        # no roots anymore (variable RD) and the rootzone shifts back to its
-        # initial depth (params.RDI). As a result the amount of water in the
-        # initial rooting depth and the unrooted layer must be redistributed. 
-        # Note that his is a rather artificial solution resulting from the fact
-        # that the rooting depth is user as to define a layer in the WOFOST
-        # water balance.
-        if self.rooted_layer_needs_reset is True:
-            self._reset_rootzone()
-
-        # calculation of new amount of soil moisture in rootzone by root growth
-        RD = self._determine_rooting_depth()            
-        if (RD - self.RDold) > 0.001:
-            # water added to root zone by root growth, in cm
-            WDR = s.WLOW * (RD - self.RDold)/(self.RDM - self.RDold)
-            s.WLOW -= WDR
-
-            # total water addition to rootzone by root growth
-            s.WDRT += WDR
-            # amount of soil moisture in extended rootzone
-            s.W += WDR
+        # First get the actual rooting depth
+        RD = self._determine_rooting_depth()
+        RDchange = RD - self.RDold
+        self._redistribute_water(RDchange)
 
         # mean soil moisture content in rooted zone
         s.SM = s.W/RD
+
+        # Accumulate days since oxygen stress, but only if a crop is present
+        if s.SM >= (p.SM0 - p.CRAIRC):  # and self.in_crop_cycle:
+            s.DSOS += 1
+        else:
+            s.DSOS = 0
+
         # save rooting depth
         self.RDold = RD
 
@@ -498,19 +554,21 @@ class WaterbalanceFD(SimulationObject):
 
         # Checksums waterbalance for systems without groundwater
         # for rootzone (WBALRT) and whole system (WBALTT)
-        s.WBALRT = s.TOTINF + s.WI + s.WDRT - s.EVST - s.WTRAT - s.PERCT - s.W
-        s.WBALTT = (p.SSI + s.RAINT + s.TOTIRR + s.WI - s.W + s.WLOWI - 
-                    s.WLOW - s.WTRAT- s.EVWT - s.EVST - s.TSR - s.LOSST - s.SS)
+        # The sum of all increments made are added to ensure a closing waterbalance
+        s.WBALRT = s.TOTINF + s.WI + s.WDRT - s.EVST - s.WTRAT - s.PERCT - s.W + sum(self._increments_W)
+        s.WBALTT = (s.SSI + s.RAINT + s.TOTIRR + s.WI - s.W + sum(self._increments_W) +
+                    s.WLOWI - s.WLOW - s.WTRAT - s.EVWT - s.EVST - s.TSR - s.LOSST - s.SS)
+
         if abs(s.WBALRT) > 0.0001:
             msg = "Water balance for root zone does not close."
             raise exc.WaterBalanceError(msg)
 
         if abs(s.WBALTT) > 0.0001:
             msg = "Water balance for complete soil profile does not close.\n"
-            msg += ("Total INIT + IN:   %f\n" % (s.WI+s.WLOWI+p.SSI+s.TOTIRR+
+            msg += ("Total INIT + IN:   %f\n" % (s.WI + s.WLOWI + s.SSI + s.TOTIRR +
                                                  s.RAINT))
-            msg += ("Total FINAL + OUT: %f\n" % (s.W+s.WLOW+s.SS+s.EVWT+s.EVST+
-                                                 s.WTRAT+s.TSR+s.LOSST))
+            msg += ("Total FINAL + OUT: %f\n" % (s.W + s.WLOW + s.SS + s.EVWT + s.EVST +
+                                                 s.WTRAT + s.TSR + s.LOSST))
             raise exc.WaterBalanceError(msg)
         
         # Run finalize on the subSimulationObjects
@@ -518,48 +576,90 @@ class WaterbalanceFD(SimulationObject):
     
     def _determine_rooting_depth(self):
         """Determines appropriate use of the rooting depth (RD)
+
+        This function includes the logic to determine the depth of the upper (rooted)
+        layer of the water balance. See the comment in the code for a detailed description.
         """
-        p = self.params
+        if "RD" in self.kiosk:
+            return self.kiosk["RD"]
+        else:
+            # Hold RD at default value
+            return self.DEFAULT_RD
 
-        if self.in_crop_cycle is False:
+    def _redistribute_water(self, RDchange):
+        """Redistributes the water between the root zone and the lower zone.
 
-            # Crop finished
-            if  "RD" in self.kiosk:
-                # Only happens at the final simulation cycle when value for
-                # SM still has to be computed. This also applies that a reset
-                # of the root zone layer is needed in the next update cycle.
-                RD = self.kiosk["RD"]
-                self.rooted_layer_needs_reset = True
-            else:
-                # not in crop cycle hold RD at initial value RDI
-                RD = p.RDI
-            
-        else: # In cropping season
-            RD = self.kiosk["RD"]
-            
-        return RD
-    
-    def _reset_rootzone(self):
+        :param RDchange: Change in root depth [cm] positive for downward growth,
+                         negative for upward growth
+
+        Redistribution of water is needed when roots grow during the growing season
+        and when the crop is finished and the root zone shifts back from the crop rooted
+        depth to the default depth of the upper (rooted) layer of the water balance.
+        Or when the initial rooting depth of a crop is different from the default one used
+        by the water balance module (10 cm)
+        """
         s = self.states
         p = self.params
         
-        self.rooted_layer_needs_reset = False
-        
-        # water added to the subsoil by root zone reset
-        WDR = s.W * (self.RDold - p.RDI)/(self.RDold)
-        s.WLOW += WDR
+        WDR = 0.
+        if RDchange > 0.001:
+            # roots grow down by more than 0.001 cm
+            # move water from previously unrooted zone and add to new rooted zone
+            WDR = s.WLOW * RDchange/(p.RDMSOL - self.RDold)
+            # Take minimum of WDR and WLOW to avoid negative WLOW due to rounding
+            WDR = min(s.WLOW, WDR)
+        else:
+            # roots disappear upwards by more than 0.001 cm (especially when crop disappears)
+            # move water from previously rooted zone and add to new unrooted zone
+            WDR = s.W * RDchange/self.RDold
+
+        if WDR != 0.:
+            # reduce amount of water in subsoil
+            s.WLOW -= WDR
+            # increase amount of water in root zone
+            s.W += WDR
+            # total water add to rootzone by root zone reset
+            s.WDRT += WDR
 
-        # total water subtracted from, rootzone by root zone reset
-        s.WDRT -= WDR
-        # amount of soil moisture in new resetted rootzone
-        s.W -= WDR
-                
-        
     def _on_CROP_START(self):
         self.in_crop_cycle = True
-        
+        self.rooted_layer_needs_reset = True
+
     def _on_CROP_FINISH(self):
         self.in_crop_cycle = False
+        self.rooted_layer_needs_reset = True
+
+    def _on_IRRIGATE(self, amount, efficiency):
+        self._RIRR = amount * efficiency
+
+    def _set_variable_SM(self, nSM):
+        """Force the model states based on the given soil moisture value.
+
+        This implies that besides the root-zone soil content, also the
+        water available in the root zone (W) must be updated since SM
+        is derived from W.
+
+        Further, the increment made to W is added to self._increments_W
+        in order to ensure that the water balance still closes.
+        """
+        s = self.states
+
+        # old values
+        oSM = s.SM
+        oW = s.W
+        # new values
+        nW = nSM/oSM * s.W
+
+        # update states
+        s.W = nW
+        s.SM = nSM
+        s.WWLOW = s.WLOW + s.W
+
+        # Store increments on W
+        self._increments_W.append(nW - oW)
+
+        # Return increments on all variables
+        return {"W": nW - oW, "SM": nSM - oSM}
 
 
 class WaterbalanceFDSnow(SimulationObject):
@@ -576,14 +676,40 @@ class WaterbalanceFDSnow(SimulationObject):
     waterbalance = Instance(SimulationObject)
     snowcover = Instance(SimulationObject)
 
+    # use observed snow depth
+    use_observed_snow_depth = Bool(False)
+    # Helper variable for observed snow depth
+    _SNOWDEPTH = Float()
+
+    class StateVariables(StatesTemplate):
+        SNOWDEPTH = Float()
+
     def initialize(self, day, kiosk, parvalues):
         self.waterbalance = WaterbalanceFD(day, kiosk, parvalues)
-        self.snowcover = SnowMAUS(day, kiosk, parvalues)
+
+        # Determine of observed or simulated snow depth should be used
+        if "ISNOWSRC" not in parvalues:
+            msg = "Parameter for selecting observed(0)/simulated(1) snow depth ('ISNOWSRC') missing!"
+            raise exc.ParameterError(msg)
+        else:
+            self.use_observed_snow_depth = True if parvalues["ISNOWSRC"] == 0 else False
+
+        if self.use_observed_snow_depth:
+            self.states = self.StateVariables(kiosk, SNOWDEPTH=0.)
+        else:
+            self.snowcover = SnowMAUS(day, kiosk, parvalues)
 
     def calc_rates(self, day, drv):
         self.waterbalance.calc_rates(day, drv)
-        self.snowcover.calc_rates(day, drv)
+        if self.use_observed_snow_depth:
+            self._SNOWDEPTH = drv.SNOWDEPTH
+        else:
+            self.snowcover.calc_rates(day, drv)
 
-    def integrate(self, day):
-        self.waterbalance.integrate(day)
-        self.snowcover.integrate(day)
+    @prepare_states
+    def integrate(self, day, delt=1.0):
+        self.waterbalance.integrate(day, delt)
+        if self.use_observed_snow_depth:
+            self.states.SNOWDEPTH = self._SNOWDEPTH
+        else:
+            self.snowcover.integrate(day, delt)
