@@ -1,35 +1,37 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2004-2014 Alterra, Wageningen-UR
-# Allard de Wit (allard.dewit@wur.nl), April 2014
+# Copyright (c) 2004-2016 Alterra, Wageningen-UR
+# Allard de Wit (allard.dewit@wur.nl), October 2016
 """Miscellaneous utilities for PCSE
 """
 import os, sys
 import datetime
 import copy
-from math import log10, cos, sin, asin, sqrt, exp
+import platform
+import tempfile
+import logging
+from math import log10, cos, sin, asin, sqrt, exp, pi, radians
 from collections import namedtuple
 from bisect import bisect_left
-try:
-    # support both python2 and python3
-    from collections import MutableMapping
-except ImportError:
-    from UserDict import DictMixin as MutableMapping
 import textwrap
 import sqlite3
-import pdb
-
-
-import numpy as np
+from collections import Iterable
 
 from . import exceptions as exc
+from .traitlets import TraitType
 
 Celsius2Kelvin = lambda x: x + 273.16
 hPa2kPa = lambda x: x/10.
+
 # Saturated Vapour pressure [kPa] at temperature temp [C]
 SatVapourPressure = lambda temp: 0.6108 * exp((17.27 * temp) / (237.3 + temp))
 
-def reference_ET(day, LAT, ELEV, TMIN, TMAX, AVRAD, VAP, WIND2,
-                 ANGSTA, ANGSTB, ETMODEL="PM"):
+# Named tuple for returning results of ASTRO
+astro_nt = namedtuple("AstroResults", "DAYL, DAYLP, SINLD, COSLD, DIFPP, "
+                                      "ATMTR, DSINBE, ANGOT")
+
+
+def reference_ET(DAY, LAT, ELEV, TMIN, TMAX, IRRAD, VAP, WIND,
+                 ANGSTA, ANGSTB, ETMODEL="PM", **kwargs):
     """Calculates reference evapotranspiration values E0, ES0 and ET0.
 
     The open water (E0) and bare soil evapotranspiration (ES0) are calculated with
@@ -39,14 +41,14 @@ def reference_ET(day, LAT, ELEV, TMIN, TMAX, AVRAD, VAP, WIND2,
 
     Input variables::
 
-        day     -  Python datetime.date object                      -
+        DAY     -  Python datetime.date object                      -
         LAT     -  Latitude of the site                          degrees
         ELEV    -  Elevation above sea level                        m
         TMIN    -  Minimum temperature                              C
         TMAX    -  Maximum temperature                              C
-        AVRAD   -  Daily shortwave radiation                     J m-2 d-1
+        IRRAD   -  Daily shortwave radiation                     J m-2 d-1
         VAP     -  24 hour average vapour pressure                 hPa
-        WIND2   -  24 hour average windspeed at 2 meter            m/s
+        WIND    -  24 hour average windspeed at 2 meter            m/s
         ANGSTA  -  Empirical constant in Angstrom formula           -
         ANGSTB  -  Empirical constant in Angstrom formula           -
         ETMODEL -  Indicates if the canopy reference ET should     PM|P
@@ -104,15 +106,15 @@ def reference_ET(day, LAT, ELEV, TMIN, TMAX, AVRAD, VAP, WIND2,
         msg = "Variable ETMODEL can have values 'PM'|'P' only."
         raise RuntimeError(msg)
 
-    E0, ES0, ET0 = penman(day, LAT, ELEV, TMIN, TMAX, AVRAD, VAP, WIND2,
+    E0, ES0, ET0 = penman(DAY, LAT, ELEV, TMIN, TMAX, IRRAD, VAP, WIND,
                           ANGSTA, ANGSTB)
     if ETMODEL == "PM":
-        ET0 = penman_monteith(day, LAT, ELEV, TMIN, TMAX, AVRAD, VAP, WIND2)
+        ET0 = penman_monteith(DAY, LAT, ELEV, TMIN, TMAX, IRRAD, VAP, WIND)
 
     return E0, ES0, ET0
 
 
-def penman(day, LAT, ELEV, TMIN, TMAX, AVRAD, VAP, WIND2, ANGSTA, ANGSTB):
+def penman(DAY, LAT, ELEV, TMIN, TMAX, AVRAD, VAP, WIND2, ANGSTA, ANGSTB):
     """Calculates E0, ES0, ET0 based on the Penman model.
     
      This routine calculates the potential evapo(transpi)ration rates from
@@ -123,7 +125,7 @@ def penman(day, LAT, ELEV, TMIN, TMAX, AVRAD, VAP, WIND2, ANGSTA, ANGSTB):
 
     Input variables::
     
-        day     -  Python datetime.date object                                    -      
+        DAY     -  Python datetime.date object                                    -
         LAT     -  Latitude of the site                        degrees   
         ELEV    -  Elevation above sea level                      m      
         TMIN    -  Minimum temperature                            C
@@ -145,7 +147,7 @@ def penman(day, LAT, ELEV, TMIN, TMAX, AVRAD, VAP, WIND2, ANGSTA, ANGSTB):
     # latent heat of evaporation of water (J/kg=J/mm)
     # Stefan Boltzmann constant (in J/m2/d/K4, e.g multiplied by 24*60*60)
     PSYCON = 0.67; REFCFW = 0.05; REFCFS = 0.15; REFCFC = 0.25
-    LHVAP = 2.45E6; STBC = 4.9E-3
+    LHVAP = 2.45E6; STBC =  5.670373E-8 * 24*60*60 # (=4.9E-3)
 
     # preparatory calculations
     # mean daily temperature and temperature difference (Celsius)
@@ -174,7 +176,7 @@ def penman(day, LAT, ELEV, TMIN, TMAX, AVRAD, VAP, WIND2, ANGSTA, ANGSTB):
     # where RI/RA is the atmospheric transmission obtained by a CALL
     # to ASTRO:
 
-    r = astro(day, LAT, AVRAD)
+    r = astro(DAY, LAT, AVRAD)
     RELSSD = limit(0., 1., (r.ATMTR-abs(ANGSTA))/abs(ANGSTB))
 
     # Terms in Penman formula, for water, soil and canopy
@@ -188,23 +190,23 @@ def penman(day, LAT, ELEV, TMIN, TMAX, AVRAD, VAP, WIND2, ANGSTA, ANGSTB):
     RNC = (AVRAD*(1.-REFCFC)-RB)/LHVAP
 
     # evaporative demand of the atmosphere (mm/d)
-    EA = 0.26 * max(0.,(SVAP-VAP)) * (0.5+BU*WIND2)
+    EA  = 0.26 * max(0.,(SVAP-VAP)) * (0.5+BU*WIND2)
     EAC = 0.26 * max(0.,(SVAP-VAP)) * (1.0+BU*WIND2)
 
     # Penman formula (1948)
-    E0 = (DELTA*RNW+GAMMA*EA)/(DELTA+GAMMA)
-    ES0 = (DELTA*RNS+GAMMA*EA)/(DELTA+GAMMA)
+    E0  = (DELTA*RNW+GAMMA*EA)/(DELTA+GAMMA)
+    ES0 = (DELTA*RNS+GAMMA*EA)/(DELTA+GAMMA) 
     ET0 = (DELTA*RNC+GAMMA*EAC)/(DELTA+GAMMA)
 
     # Ensure reference evaporation >= 0.
-    E0 = max(0., E0)
+    E0  = max(0., E0)
     ES0 = max(0., ES0)
     ET0 = max(0., ET0)
     
     return E0, ES0, ET0
 
 
-def penman_monteith(day, LAT, ELEV, TMIN, TMAX, AVRAD, VAP, WIND2):
+def penman_monteith(DAY, LAT, ELEV, TMIN, TMAX, AVRAD, VAP, WIND2):
     """Calculates reference ET0 based on the Penman-Monteith model.
 
      This routine calculates the potential evapotranspiration rate from
@@ -215,7 +217,7 @@ def penman_monteith(day, LAT, ELEV, TMIN, TMAX, AVRAD, VAP, WIND2):
 
     Input variables::
 
-        day   -  Python datetime.date object                   -
+        DAY   -  Python datetime.date object                   -
         LAT   -  Latitude of the site                        degrees
         ELEV  - Elevation above sea level                      m
         TMIN  - Minimum temperature                            C
@@ -247,8 +249,8 @@ def penman_monteith(day, LAT, ELEV, TMIN, TMAX, AVRAD, VAP, WIND2):
     # Vapour pressure to kPa
     VAP = hPa2kPa(VAP)
 
-    # atmospheric pressure (kPa)
-    T = Celsius2Kelvin(TMPA)
+    # atmospheric pressure at standard temperature of 293K (kPa)
+    T = 293.0
     PATM = 101.3 * pow((T - (0.0065*ELEV))/T, 5.26)
 
     # psychrometric constant (kPa/Celsius)
@@ -273,9 +275,9 @@ def penman_monteith(day, LAT, ELEV, TMIN, TMAX, AVRAD, VAP, WIND2):
     STB_TMIN = STBC * pow(Celsius2Kelvin(TMIN), 4)
     RNL_TMP = ((STB_TMAX + STB_TMIN) / 2.) * (0.34 - 0.14 * sqrt(VAP))
 
-    # Clear Sky radiation [J/m2/day] from Angot TOA radiation
+    # Clear Sky radiation [J/m2/DAY] from Angot TOA radiation
     # the latter is found through a call to astro()
-    r = astro(day, LAT, AVRAD)
+    r = astro(DAY, LAT, AVRAD)
     CSKYRAD = (0.75 + (2e-05 * ELEV)) * r.ANGOT
 
     if CSKYRAD > 0:
@@ -283,7 +285,7 @@ def penman_monteith(day, LAT, ELEV, TMIN, TMAX, AVRAD, VAP, WIND2):
         RNL = RNL_TMP * (1.35 * (AVRAD/CSKYRAD) - 0.35)
 
         # radiative evaporation equivalent for the reference surface
-        # [mm/day]
+        # [mm/DAY]
         RN = ((1-REFCFC) * AVRAD - RNL)/LHVAP
 
         # aerodynamic evaporation equivalent [mm/day]
@@ -300,7 +302,7 @@ def penman_monteith(day, LAT, ELEV, TMIN, TMAX, AVRAD, VAP, WIND2):
 
     return ET0
 
-#-------------------------------------------------------------------------------
+
 def check_angstromAB(xA, xB):
     """Routine checks validity of Angstrom coefficients.
     
@@ -357,13 +359,29 @@ def ea_from_tdew(tdew):
     # Raise exception:
     if (tdew < -95.0 or tdew > 65.0):
         # Are these reasonable bounds?
-        raise ValueError, 'tdew=%g is not in range -95 to +60 deg C' % tdew
+        msg = 'tdew=%g is not in range -95 to +60 deg C' % tdew
+        raise ValueError(msg)
 
     tmp = (17.27 * tdew) / (tdew + 237.3)
     ea = 0.6108 * exp(tmp)
     return ea
 
 
+def vap_from_relhum(rh, temp):
+    """Compute the actual vapour pressure from the relative humidity at given temperaure
+
+    :param rh: relative humidity as a percentage
+    :param temp: temperature corresponding to the relative humidity computation
+    :return: vapour pressure in kPa
+    """
+
+    if not 0 <= rh <= 100:
+        msg = "Relative humidity should be between 0 and 100"
+        raise RuntimeError(msg)
+
+    return SatVapourPressure(temp) * rh * 0.01
+
+
 def angstrom(day, latitude, ssd, cA, cB):
     """Compute global radiation using the Angstrom equation.
     
@@ -387,23 +405,19 @@ def doy(day):
     """Converts a date or datetime object to day-of-year (Jan 1st = doy 1)
     """
     # Check if day is a date or datetime object
-    if isinstance(day, datetime.date):
-        pass
-    elif isinstance(day, datetime.datetime):
-        day = day.date()
+    if isinstance(day, (datetime.date, datetime.datetime)):
+        return day.timetuple().tm_yday
     else:
         msg = "Parameter day is not a date or datetime object."
         raise RuntimeError(msg)
 
-    return (day-datetime.date(day.year,1,1)).days + 1
-
 
 def limit(min, max, v):
     """limits the range of v between min and max
     """
 
     if min > max:
-        raise RuntimeError("Min value larger than max")
+        raise RuntimeError("Min value (%f) larger than max (%f)" % (min, max))
     
     if v < min:       # V below range: return min
         return min
@@ -443,26 +457,19 @@ def daylength(day, latitude, angle=-4, _cache={}):
         pass
     
     # constants
-    RAD = 0.0174533
-    PI = 3.1415926
-
-    # map python functions to capitals
-    SIN = sin
-    COS = cos
-    ASIN = asin
-    REAL = float
+    RAD = radians(1.)
 
     # calculate daylength
     ANGLE = angle
     LAT = latitude
-    DEC = -ASIN(SIN(23.45*RAD)*COS(2.*PI*(REAL(IDAY)+10.)/365.))
-    SINLD = SIN(RAD*LAT)*SIN(DEC)
-    COSLD = COS(RAD*LAT)*COS(DEC)
-    AOB   = (-SIN(ANGLE*RAD)+SINLD)/COSLD
+    DEC = -asin(sin(23.45*RAD)*cos(2.*pi*(float(IDAY)+10.)/365.))
+    SINLD = sin(RAD*LAT)*sin(DEC)
+    COSLD = cos(RAD*LAT)*cos(DEC)
+    AOB = (-sin(ANGLE*RAD)+SINLD)/COSLD
 
     # daylength
     if abs(AOB) <= 1.0:
-        DAYLP = 12.0*(1.+2.*ASIN((-SIN(ANGLE*RAD)+SINLD)/COSLD)/PI)
+        DAYLP = 12.0*(1.+2.*asin((-sin(ANGLE*RAD)+SINLD)/COSLD)/pi)
     elif AOB > 1.0:
         DAYLP = 24.0
     else:
@@ -525,26 +532,17 @@ def astro(day, latitude, radiation, _cache={}):
         pass
 
     # constants
-    RAD = 0.0174533
-    PI = 3.1415926
+    RAD = radians(1.)
     ANGLE = -4.
 
-    # map python functions to capitals
-    SIN = sin
-    COS = cos
-    ASIN = asin
-    REAL = float
-    SQRT = sqrt
-    ABS = abs
-
     # Declination and solar constant for this day
-    DEC = -ASIN(SIN(23.45*RAD)*COS(2.*PI*(REAL(IDAY)+10.)/365.))
-    SC  = 1370.*(1.+0.033*COS(2.*PI*REAL(IDAY)/365.))
+    DEC = -asin(sin(23.45*RAD)*cos(2.*pi*(float(IDAY)+10.)/365.))
+    SC  = 1370.*(1.+0.033*cos(2.*pi*float(IDAY)/365.))
 
     # calculation of daylength from intermediate variables
     # SINLD, COSLD and AOB
-    SINLD = SIN(RAD*LAT)*SIN(DEC)
-    COSLD = COS(RAD*LAT)*COS(DEC)
+    SINLD = sin(RAD*LAT)*sin(DEC)
+    COSLD = cos(RAD*LAT)*cos(DEC)
     AOB = SINLD/COSLD
 
     # For very high latitudes and days in summer and winter a limit is
@@ -553,11 +551,11 @@ def astro(day, latitude, radiation, _cache={}):
 
     # Calculate solution for base=0 degrees
     if abs(AOB) <= 1.0:
-        DAYL  = 12.0*(1.+2.*ASIN(AOB)/PI)
+        DAYL  = 12.0*(1.+2.*asin(AOB)/pi)
         # integrals of sine of solar height
-        DSINB  = 3600.*(DAYL*SINLD+24.*COSLD*SQRT(1.-AOB**2)/PI)
+        DSINB  = 3600.*(DAYL*SINLD+24.*COSLD*sqrt(1.-AOB**2)/pi)
         DSINBE = 3600.*(DAYL*(SINLD+0.4*(SINLD**2+COSLD**2*0.5))+
-                 12.*COSLD*(2.+3.*0.4*SINLD)*SQRT(1.-AOB**2)/PI)
+                 12.*COSLD*(2.+3.*0.4*SINLD)*sqrt(1.-AOB**2)/pi)
     else:
         if AOB >  1.0: DAYL = 24.0
         if AOB < -1.0: DAYL = 0.0
@@ -566,9 +564,9 @@ def astro(day, latitude, radiation, _cache={}):
         DSINBE = 3600.*(DAYL*(SINLD+0.4*(SINLD**2+COSLD**2*0.5)))
 
     # Calculate solution for base=-4 (ANGLE) degrees
-    AOB_CORR = (-SIN(ANGLE*RAD)+SINLD)/COSLD
+    AOB_CORR = (-sin(ANGLE*RAD)+SINLD)/COSLD
     if abs(AOB_CORR) <= 1.0:
-        DAYLP = 12.0*(1.+2.*ASIN(AOB_CORR)/PI)
+        DAYLP = 12.0*(1.+2.*asin(AOB_CORR)/pi)
     elif AOB_CORR > 1.0:
         DAYLP = 24.0
     elif AOB_CORR < -1.0:
@@ -593,16 +591,13 @@ def astro(day, latitude, radiation, _cache={}):
         FRDIF = 1.
 
     DIFPP = FRDIF*ATMTR*0.5*SC
-    
-    # Pack return values in namedtuple, add to cache and return
-    astro_nt = namedtuple("AstroResults","DAYL, DAYLP, SINLD, COSLD, DIFPP, "
-                                         "ATMTR, DSINBE, ANGOT")
+
     retvalue = astro_nt(DAYL, DAYLP, SINLD, COSLD, DIFPP, ATMTR, DSINBE, ANGOT)
     _cache[(IDAY, LAT, AVRAD)] = retvalue
 
     return retvalue
 
-#-------------------------------------------------------------------------------
+
 class Afgen(object):
     """Emulates the AFGEN function in WOFOST.
     
@@ -642,7 +637,7 @@ class Afgen(object):
         n = len(x_list)
         
         # Check if x range is ascending continuously
-        rng = range(1, n)
+        rng = list(range(1, n))
         x_asc = [True if (x_list[i] > x_list[i-1]) else False for i in rng]
         
         # Check for breaks in the series where the ascending sequence stops.
@@ -671,9 +666,9 @@ class Afgen(object):
         self.unit = unit
 
         x_list, y_list = self._check_x_ascending(tbl_xy)
-        x_list = self.x_list = map(float, x_list)
-        y_list = self.y_list = map(float, y_list)
-        intervals = zip(x_list, x_list[1:], y_list, y_list[1:])
+        x_list = self.x_list = list(map(float, x_list))
+        y_list = self.y_list = list(map(float, y_list))
+        intervals = list(zip(x_list, x_list[1:], y_list, y_list[1:]))
         self.slopes = [(y2 - y1)/(x2 - x1) for x1, x2, y1, y2 in intervals]
 
     def __call__(self, x):
@@ -692,103 +687,18 @@ class Afgen(object):
 
         return v
 
-#-------------------------------------------------------------------------------
-class Afgen2(object):
-    """Emulates the AFGEN function in TTUTIL with Numpy.interp
-    
-    :param tbl_xy: List or array of XY value pairs describing the function
-        the X values should be mononically increasing.
-    :param unit: The interpolated values is returned with given
-        `unit <http://pypi.python.org/pypi/Unum/4.1.0>`_ assigned.
-    
-    Returns the interpolated value provided with the 
-    absicca value at which the interpolation should take place.
-    
-    example::
-    
-        >>> tbl_xy = [0,0,1,1,5,10]
-        >>> f =  Afgen(tbl_xy)
-        >>> f(0.5)
-        0.5
-        >>> f(1.5)
-        2.125
-        >>> f(5)
-        10.0
-        >>> f(6)
-        10.0
-        >>> f(-1)
-        0.0
-    """
-    
-    def __init__(self, tbl_xy, unit=None):
-    
-        x = tbl_xy[0::2]
-        y = tbl_xy[1::2]
-        
-        # Determine if there are empty pairs in tbl_xy by searching
-        # for the point where x stops monotonically increasing
-        xpos = np.arange(1, len(x))
-        ibreak = False
-        for i in xpos:
-            if x[i] <= x[i-1]:
-                ibreak = True
-                break
-        
-        if ibreak is True:
-            x = x[0:i]
-            y = y[0:i]
-        
-        self.x = x
-        self.y = y
-        self.unit = unit
-    
-    def __call__(self, p):
-        
-        v = np.interp(p, self.x, self.y)
-        
-        # if a unum unit is defined, multiply with a unit
-        if self.unit is not None:
-            v *= self.unit
-        return float(v)
-    
-    def __str__(self):
-        msg = "AFGEN interpolation over (X,Y) pairs:\n"
-        for (x,y) in zip(self.x, self.y):
-            msg += ("(%f,%f)\n " % (x,y))
-        msg += "\n"
-        if self.unit is not None:
-             msg += "Return value as unit: %s" % self.unit
-        return msg
 
-#-------------------------------------------------------------------------------
-class Chainmap(MutableMapping):
-    """Combine multiple mappings for sequential lookup.
+class AfgenTrait(TraitType):
+    """An AFGEN table trait"""
+    default_value = Afgen([0,0,1,1])
+    into_text = "An AFGEN table of XY pairs"
 
-    For example, to emulate Python's normal lookup sequence:
-
-        import __builtin__
-        pylookup = Chainmap(locals(), globals(), vars(__builtin__))
-    
-    recipe taken from http://code.activestate.com/recipes/305268/
-    Available natively in the connections module in python 3.3
-    """
-
-    def __init__(self, *maps):
-        self._maps = maps
-
-    def __getitem__(self, key):
-        for mapping in self._maps:
-            try:
-                return mapping[key]
-            except KeyError:
-                pass
-        raise KeyError(key)
-        
-    def keys(self):
-        k = []
-        for mapping in self._maps:
-            return k.extend(mapping.keys)
-        return k
+    def validate(self, obj, value):
+        if isinstance(value, Afgen):
+           return value
+        elif isinstance(value, Iterable):
+           return Afgen(value)
+        self.error(obj, value)
 
 
 def merge_dict(d1, d2, overwrite=False):
@@ -855,8 +765,9 @@ class ConfigurationLoader(object):
         # Load file using execfile
         try:
             loc = {}
-            execfile(model_config_file, {}, loc)
-        except Exception, e:
+            bytecode = compile(open(model_config_file).read(), model_config_file, 'exec')
+            exec(bytecode, {}, loc)
+        except Exception as e:
             msg = "Failed to load configuration from file '%s' due to: %s"
             msg = msg % (model_config_file, e)
             raise exc.PCSEError(msg)
@@ -870,7 +781,7 @@ class ConfigurationLoader(object):
                     self.description += "\n"
 
         # Loop through the attributes in the configuration file
-        for key, value in loc.items():
+        for key, value in list(loc.items()):
             if key.isupper():
                 self.defined_attr.append(key)
                 setattr(self, key, value)
@@ -907,9 +818,9 @@ def is_a_month(day):
         if (day == datetime.date(day.year, day.month+1, 1) - \
                    datetime.timedelta(days=1)):
             return True
-
     return False
 
+
 def is_a_week(day, weekday=0):
     """Default weekday is Monday. Monday is 0 and Sunday is 6"""
     if day.weekday() == weekday:
@@ -936,9 +847,9 @@ def is_a_dekad(day):
         elif (day == datetime.date(day.year, day.month+1, 1) -
                      datetime.timedelta(days=1)):
             return True
-
     return False
 
+
 def load_SQLite_dump_file(dump_file_name, SQLite_db_name):
     """Build an SQLite database <SQLite_db_name> from dump file <dump_file_name>.
     """
@@ -949,3 +860,249 @@ def load_SQLite_dump_file(dump_file_name, SQLite_db_name):
     con = sqlite3.connect(SQLite_db_name)
     con.executescript(str_sql_dump)
     con.close()
+
+
+def safe_float(x):
+    """Returns the value of x converted to float, if fails return None.
+    """
+    try:
+        return float(x)
+    except (ValueError, TypeError):
+        return None
+
+
+def check_date(indate):
+        """Check representations of date and try to force into a datetime.date
+
+        The following formats are supported:
+
+        1. a date object
+        2. a datetime object
+        3. a string of the format YYYYMMDD
+        4. a string of the format YYYYDDD
+        5. a string of the format YYYY-MM-DD
+        """
+
+        import datetime as dt
+        if isinstance(indate, dt.datetime):
+            return indate.date()
+        elif isinstance(indate, dt.date):
+            return indate
+        elif isinstance(indate, str):
+            skey = indate.strip()
+            l = len(skey)
+            if l==8:
+                # assume YYYYMMDD
+                dkey = dt.datetime.strptime(skey,"%Y%m%d")
+                return dkey.date()
+            elif l==7:
+                # assume YYYYDDD
+                dkey = dt.datetime.strptime(skey,"%Y%j")
+                return dkey.date()
+            elif l==10:
+                # assume YYYY-MM-DD
+                dkey = dt.datetime.strptime(skey,"%Y-%m-%d")
+                return dkey.date()
+            else:
+                msg = "Input value not recognized as date: %s"
+                raise KeyError(msg % indate)
+        else:
+            msg = "Input value not recognized as date: %s"
+            raise KeyError(msg % indate)
+
+
+class DummySoilDataProvider(dict):
+    """This class is to provide some dummy soil parameters for potential production simulation.
+
+    Simulation of potential production levels is independent of the soil. Nevertheless, the model
+    does not some parameter values. This data provider provides some hard coded parameter values for
+    this situation.
+    """
+    _defaults = {"SMFCF":0.3,
+                 "SM0":0.4,
+                 "SMW":0.1,
+                 "RDMSOL":120,
+                 "CRAIRC":0.06,
+                 "K0":10.,
+                 "SOPE":10.,
+                 "KSUB":10.}
+
+    def __init__(self):
+        self.update(self._defaults)
+
+
+def version_tuple(v):
+    """Creates a version tuple from a version string for consistent comparison of versions.
+
+    Conversion to tuples is needed because version '2.12.9' is higher then '2.7.8' however::
+
+    >>> '2.12.9' > '2.7.8'
+    False
+
+    Instead we need:
+
+    >>> version_tuple('2.12.9') > version_tuple('2.7.8')
+    True
+    """
+    return tuple(map(int, (v.split("."))))
+
+
+class _GenericSiteDataProvider(dict):
+    """Generic Site data provider
+
+    It just checks the values provided as keywords given the _defaults and _required values
+
+    _defaults = {"VARNAME": (default, {maxvalue, minvalue}, type),
+                }
+    _required = ["VARNAME"]
+    """
+
+    def __init__(self, **kwargs):
+        dict.__init__(self)
+
+        for par_name, (default_value, par_range, par_conversion) in self._defaults.items():
+            if par_name not in kwargs:
+                # parameter was not provided, use the default if possible
+                if par_name in self._required:
+                    msg = "Value for parameter '%s' must be provided!" % par_name
+                    raise exc.PCSEError(msg)
+                else:
+                    v = default_value
+            else:
+                # parameter was provided, check value for type and range
+                v = par_conversion(kwargs.pop(par_name))
+                if isinstance(par_range, set):
+                    if v not in par_range:
+                        msg = "Value for parameter '%s' can only have values: %s" % (par_name, par_range)
+                        raise exc.PCSEError(msg)
+                else:
+                    if not (par_range[0] <= v <= par_range[1]):
+                        msg = "Value for parameter '%s' out of range %s-%s" % \
+                              (par_name, par_range[0], par_range[1])
+                        raise exc.PCSEError(msg)
+            self[par_name] = v
+
+        # Check if kwargs is empty
+        if kwargs:
+            msg = "Unknown parameter values provided to WOFOSTSiteDataProvider: %s" % kwargs
+            print(msg)
+
+
+class WOFOST72SiteDataProvider(_GenericSiteDataProvider):
+    """Site data provider for WOFOST 7.2.
+
+    Site specific parameters for WOFOST 7.2 can be provided through this data provider as well as through
+    a normal python dictionary. The sole purpose of implementing this data provider is that the site
+    parameters for WOFOST are documented, checked and that sensible default values are given.
+
+    The following site specific parameter values can be set through this data provider::
+
+        - IFUNRN    Indicates whether non-infiltrating fraction of rain is a function of storm size (1)
+                    or not (0). Default 0
+        - NOTINF    Maximum fraction of rain not-infiltrating into the soil [0-1], default 0.
+        - SSMAX     Maximum depth of water that can be stored on the soil surface [cm]
+        - SSI       Initial depth of water stored on the surface [cm]
+        - WAV       Initial amount of water in total soil profile [cm]
+        - SMLIM     Initial maximum moisture content in initial rooting depth zone [0-1], default 0.4
+
+    Currently only the value for WAV is mandatory to specify.
+    """
+    
+    _defaults = {"IFUNRN": (0, {0, 1}, int),
+                 "NOTINF": (0, [0., 1.], float),
+                 "SSI": (0., [0., 100.], float),
+                 "SSMAX": (0., [0., 100.], float),
+                 "WAV": (None, [0., 100.], float),
+                 "SMLIM": (0.4, [0., 1.], float)}
+    _required = ["WAV"]
+
+# This is just to keep old code working
+WOFOST71SiteDataProvider = WOFOST72SiteDataProvider
+
+
+class WOFOST80SiteDataProvider(_GenericSiteDataProvider):
+    """Site data provider for WOFOST 8.0.
+
+    Site specific parameters for WOFOST 8.0 can be provided through this data provider as well as through
+    a normal python dictionary. The sole purpose of implementing this data provider is that the site
+    parameters for WOFOST are documented, checked and that sensible default values are given.
+
+    The following site specific parameter values can be set through this data provider::
+
+        - IFUNRN        Indicates whether non-infiltrating fraction of rain is a function of
+                        storm size (1) or not (0). Default 0
+        - NOTINF        Maximum fraction of rain not-infiltrating into the soil [0-1],
+                        default 0.
+        - SSMAX         Maximum depth of water that can be stored on the soil surface [cm]
+        - SSI           Initial depth of water stored on the surface [cm]
+        - WAV           Initial amount of water in total soil profile [cm]
+        - SMLIM         Initial maximum moisture content in initial rooting depth zone [0-1],
+                        default 0.4
+        - CO2           Atmospheric CO2 level (ppm), default 360.
+        - BG_N_SUPPLY   Background N supply through atmospheric deposition in kg/ha/day. Can be
+                        in the order of 25 kg/ha/year in areas with high N pollution. Default 0.0
+        - NSOILBASE     Base N amount available in the soil. This is often estimated as the nutrient
+                        left over from the previous growth cycle (surplus nutrients, crop residues
+                        or green manure).
+        - NSOILBASE_FR  Daily fraction of soil N coming available through mineralization
+        - BG_P_SUPPLY   Background P supply in kg/ha/day. Usually this is mainly through deposition
+                        of dust and an order of magnitude smaller then N deposition. Default 0.0
+        - PSOILBASE     Base P amount available in the soil.
+        - PSOILBASE_FR  Daily fraction of soil P coming available through mineralization
+        - BG_K_SUPPLY   Background P supply in kg/ha/day. Default 0.0
+        - KSOILBASE     Base K amount available in the soil
+        - KSOILBASE_FR  Daily fraction of soil K coming available through mineralization
+        - NAVAILI       Amount of N available in the pool at initialization of the system [kg/ha]
+        - PAVAILI       Amount of P available in the pool at initialization of the system [kg/ha]
+        - KAVAILI       Amount of K available in the pool at initialization of the system [kg/ha]
+
+    Currently, the parameters for initial water availability (WAV) and initial availability of
+    nutrients (NAVAILI, PAVAILI, KAVAILI) are mandatory to specify.
+    """
+
+    _defaults = {"IFUNRN": (0, {0, 1}, int),
+                 "NOTINF": (0, [0., 1.], float),
+                 "SSI": (0., [0., 100.], float),
+                 "SSMAX": (0., [0., 100.], float),
+                 "WAV": (None, [0., 100.], float),
+                 "SMLIM": (0.4, [0., 1.], float),
+                 "CO2": (360., [300., 1400.], float),
+                 "BG_N_SUPPLY": (0, (0, 0.1), float),
+                 "NSOILBASE": (0, (0, 100), float),
+                 "NSOILBASE_FR": (0.025, (0, 100), float),
+                 "BG_P_SUPPLY": (0, (0, 0.05), float),
+                 "PSOILBASE": (0, (0, 100), float),
+                 "PSOILBASE_FR": (0.025, (0, 100), float),
+                 "BG_K_SUPPLY": (0, (0, 0.05), float),
+                 "KSOILBASE": (0, (0, 100), float),
+                 "KSOILBASE_FR": (0.025, (0, 0.05), float),
+                 "NAVAILI": (None, (0, 250), float),
+                 "PAVAILI": (None, (0, 50), float),
+                 "KAVAILI": (None, (0, 250), float),
+                 }
+    _required = ["WAV", "NAVAILI", "PAVAILI", "KAVAILI"]
+
+
+def get_user_home():
+    """A reasonable platform independent way to get the user home folder.
+    If PCSE runs under a system user then return the temp directory as returned
+    by tempfile.gettempdir()
+    """
+    user_home = None
+    if platform.system() == "Windows":
+        user = os.getenv("USERNAME")
+        if user is not None:
+            user_home = os.path.expanduser("~")
+    elif platform.system() == "Linux":
+        user = os.getenv("USER")
+        if user is not None:
+            user_home = os.path.expanduser("~")
+    else:
+        msg = "Platform not recognized, using system temp directory for PCSE settings."
+        logger = logging.getLogger("pcse")
+        logger.warning(msg)
+
+    if user_home is None:
+        user_home = tempfile.gettempdir()
+
+    return user_home
